@@ -17,6 +17,22 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:element-dtsdec
+ *
+ * Digital Theatre System (DTS) audio decoder
+ * 
+ * <refsect2>
+ * <title>Example launch line</title>
+ * |[
+ * gst-launch dvdreadsrc title=1 ! mpegpsdemux ! dtsdec ! audioresample ! audioconvert ! alsasink
+ * ]| Play a DTS audio track from a dvd.
+ * |[
+ * gst-launch filesrc location=abc.dts ! dtsdec ! audioresample ! audioconvert ! alsasink
+ * ]| Decode a standalone file and play it.
+ * </refsect2>
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -28,8 +44,40 @@
 #include <gst/gst.h>
 #include <gst/audio/multichannel.h>
 
+#ifndef DTS_OLD
+#include <dca.h>
+#else
 #include <dts.h>
 
+typedef struct dts_state_s dca_state_t;
+#define DCA_MONO DTS_MONO
+#define DCA_CHANNEL DTS_CHANNEL
+#define DCA_STEREO DTS_STEREO
+#define DCA_STEREO_SUMDIFF DTS_STEREO_SUMDIFF
+#define DCA_STEREO_TOTAL DTS_STEREO_TOTAL
+#define DCA_3F DTS_3F
+#define DCA_2F1R DTS_2F1R
+#define DCA_3F1R DTS_3F1R
+#define DCA_2F2R DTS_2F2R
+#define DCA_3F2R DTS_3F2R
+#define DCA_4F2R DTS_4F2R
+#define DCA_DOLBY DTS_DOLBY
+#define DCA_CHANNEL_MAX DTS_CHANNEL_MAX
+#define DCA_CHANNEL_BITS DTS_CHANNEL_BITS
+#define DCA_CHANNEL_MASK DTS_CHANNEL_MASK
+#define DCA_LFE DTS_LFE
+#define DCA_ADJUST_LEVEL DTS_ADJUST_LEVEL
+
+#define dca_init dts_init
+#define dca_syncinfo dts_syncinfo
+#define dca_frame dts_frame
+#define dca_dynrng dts_dynrng
+#define dca_blocks_num dts_blocks_num
+#define dca_block dts_block
+#define dca_samples dts_samples
+#define dca_free dts_free
+#endif
+
 #include "gstdtsdec.h"
 
 #include <liboil/liboil.h>
@@ -45,33 +93,25 @@ GST_ELEMENT_DETAILS ("DTS audio decoder",
     "Decodes DTS audio streams",
     "Ronald Bultje <rbultje@ronald.bitfreak.net>");
 
-enum
-{
-  /* FILL ME */
-  LAST_SIGNAL
-};
-
 enum
 {
   ARG_0,
   ARG_DRC
-      /* FILL ME */
 };
-
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("audio/x-dts")
+    GST_STATIC_CAPS ("audio/x-dts;" "audio/x-private1-dts")
     );
 
-#if defined(LIBDTS_FIXED)
+#if defined(LIBDTS_FIXED) || defined(LIBDCA_FIXED)
 #define DTS_CAPS "audio/x-raw-int, " \
     "endianness = (int) " G_STRINGIFY (G_BYTE_ORDER) ", " \
     "signed = (boolean) true, " \
     "width = (int) 16, " \
     "depth = (int) 16"
 #define SAMPLE_WIDTH 16
-#elif defined(LIBDTS_DOUBLE)
+#elif defined(LIBDTS_DOUBLE) || defined(LIBDCA_DOUBLE)
 #define DTS_CAPS "audio/x-raw-float, " \
     "endianness = (int) " G_STRINGIFY (G_BYTE_ORDER) ", " \
     "width = (int) 64"
@@ -92,6 +132,7 @@ static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
 
 GST_BOILERPLATE (GstDtsDec, gst_dtsdec, GstElement, GST_TYPE_ELEMENT);
 
+static gboolean gst_dtsdec_sink_setcaps (GstPad * pad, GstCaps * caps);
 static gboolean gst_dtsdec_sink_event (GstPad * pad, GstEvent * event);
 static GstFlowReturn gst_dtsdec_chain (GstPad * pad, GstBuffer * buf);
 static GstStateChangeReturn gst_dtsdec_change_state (GstElement * element,
@@ -132,6 +173,14 @@ gst_dtsdec_class_init (GstDtsDecClass * klass)
 
   gstelement_class->change_state = gst_dtsdec_change_state;
 
+  /**
+   * GstDtsDec::drc
+   *
+   * Set to true to apply the recommended DTS dynamic range compression
+   * to the audio stream. Dynamic range compression makes loud sounds
+   * softer and soft sounds louder, so you can more easily listen
+   * to the stream without disturbing other people.
+   */
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_DRC,
       g_param_spec_boolean ("drc", "Dynamic Range Compression",
           "Use Dynamic Range Compression", FALSE, G_PARAM_READWRITE));
@@ -153,18 +202,16 @@ gst_dtsdec_class_init (GstDtsDecClass * klass)
 static void
 gst_dtsdec_init (GstDtsDec * dtsdec, GstDtsDecClass * g_class)
 {
-  /* create the sink and src pads */
-  dtsdec->sinkpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&sink_factory), "sink");
-  gst_pad_set_chain_function (dtsdec->sinkpad, gst_dtsdec_chain);
+  dtsdec->sinkpad = gst_pad_new_from_static_template (&sink_factory, "sink");
+  gst_pad_set_setcaps_function (dtsdec->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_dtsdec_sink_setcaps));
+  gst_pad_set_chain_function (dtsdec->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_dtsdec_chain));
   gst_pad_set_event_function (dtsdec->sinkpad,
       GST_DEBUG_FUNCPTR (gst_dtsdec_sink_event));
   gst_element_add_pad (GST_ELEMENT (dtsdec), dtsdec->sinkpad);
 
-  dtsdec->srcpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&src_factory), "src");
+  dtsdec->srcpad = gst_pad_new_from_static_template (&src_factory, "src");
   gst_pad_use_fixed_caps (dtsdec->srcpad);
   gst_element_add_pad (GST_ELEMENT (dtsdec), dtsdec->srcpad);
 
@@ -184,24 +231,24 @@ gst_dtsdec_channels (uint32_t flags, GstAudioChannelPosition ** pos)
       return 0;
   }
 
-  switch (flags & DTS_CHANNEL_MASK) {
-    case DTS_MONO:
+  switch (flags & DCA_CHANNEL_MASK) {
+    case DCA_MONO:
       chans = 1;
       if (tpos)
         tpos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_MONO;
       break;
-      /* case DTS_CHANNEL: */
-    case DTS_STEREO:
-    case DTS_STEREO_SUMDIFF:
-    case DTS_STEREO_TOTAL:
-    case DTS_DOLBY:
+      /* case DCA_CHANNEL: */
+    case DCA_STEREO:
+    case DCA_STEREO_SUMDIFF:
+    case DCA_STEREO_TOTAL:
+    case DCA_DOLBY:
       chans = 2;
       if (tpos) {
         tpos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
         tpos[1] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
       }
       break;
-    case DTS_3F:
+    case DCA_3F:
       chans = 3;
       if (tpos) {
         tpos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER;
@@ -209,7 +256,7 @@ gst_dtsdec_channels (uint32_t flags, GstAudioChannelPosition ** pos)
         tpos[2] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
       }
       break;
-    case DTS_2F1R:
+    case DCA_2F1R:
       chans = 3;
       if (tpos) {
         tpos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
@@ -217,7 +264,7 @@ gst_dtsdec_channels (uint32_t flags, GstAudioChannelPosition ** pos)
         tpos[2] = GST_AUDIO_CHANNEL_POSITION_REAR_CENTER;
       }
       break;
-    case DTS_3F1R:
+    case DCA_3F1R:
       chans = 4;
       if (tpos) {
         tpos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER;
@@ -226,7 +273,7 @@ gst_dtsdec_channels (uint32_t flags, GstAudioChannelPosition ** pos)
         tpos[3] = GST_AUDIO_CHANNEL_POSITION_REAR_CENTER;
       }
       break;
-    case DTS_2F2R:
+    case DCA_2F2R:
       chans = 4;
       if (tpos) {
         tpos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
@@ -235,7 +282,7 @@ gst_dtsdec_channels (uint32_t flags, GstAudioChannelPosition ** pos)
         tpos[3] = GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT;
       }
       break;
-    case DTS_3F2R:
+    case DCA_3F2R:
       chans = 5;
       if (tpos) {
         tpos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER;
@@ -245,7 +292,7 @@ gst_dtsdec_channels (uint32_t flags, GstAudioChannelPosition ** pos)
         tpos[4] = GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT;
       }
       break;
-    case DTS_4F2R:
+    case DCA_4F2R:
       chans = 6;
       if (tpos) {
         tpos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT_OF_CENTER;
@@ -260,7 +307,7 @@ gst_dtsdec_channels (uint32_t flags, GstAudioChannelPosition ** pos)
       g_warning ("dtsdec: invalid flags 0x%x", flags);
       return 0;
   }
-  if (flags & DTS_LFE) {
+  if (flags & DCA_LFE) {
     if (tpos) {
       tpos[chans] = GST_AUDIO_CHANNEL_POSITION_LFE;
     }
@@ -308,8 +355,7 @@ gst_dtsdec_sink_event (GstPad * pad, GstEvent * event)
   GstDtsDec *dtsdec = GST_DTSDEC (gst_pad_get_parent (pad));
   gboolean ret = FALSE;
 
-  GST_LOG ("Handling event of type %d timestamp %llu", GST_EVENT_TYPE (event),
-      GST_EVENT_TIMESTAMP (event));
+  GST_LOG_OBJECT (dtsdec, "%s event", GST_EVENT_TYPE_NAME (event));
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_NEWSEGMENT:{
@@ -331,14 +377,6 @@ gst_dtsdec_sink_event (GstPad * pad, GstEvent * event)
       ret = gst_pad_event_default (pad, event);
       break;
     }
-    case GST_EVENT_TAG:
-    case GST_EVENT_EOS:{
-      ret = gst_pad_event_default (pad, event);
-      break;
-    }
-    case GST_EVENT_FLUSH_START:
-      ret = gst_pad_event_default (pad, event);
-      break;
     case GST_EVENT_FLUSH_STOP:
       if (dtsdec->cache) {
         gst_buffer_unref (dtsdec->cache);
@@ -355,6 +393,24 @@ gst_dtsdec_sink_event (GstPad * pad, GstEvent * event)
   return ret;
 }
 
+static gboolean
+gst_dtsdec_sink_setcaps (GstPad * pad, GstCaps * caps)
+{
+  GstDtsDec *dts = GST_DTSDEC (gst_pad_get_parent (pad));
+  GstStructure *structure;
+
+  structure = gst_caps_get_structure (caps, 0);
+
+  if (structure && gst_structure_has_name (structure, "audio/x-private1-dts"))
+    dts->dvdmode = TRUE;
+  else
+    dts->dvdmode = FALSE;
+
+  gst_object_unref (dts);
+
+  return TRUE;
+}
+
 static void
 gst_dtsdec_update_streaminfo (GstDtsDec * dts)
 {
@@ -392,16 +448,55 @@ gst_dtsdec_handle_frame (GstDtsDec * dts, guint8 * data,
     gst_dtsdec_update_streaminfo (dts);
   }
 
+  if (dts->request_channels == DCA_CHANNEL) {
+    GstCaps *caps;
+
+    caps = gst_pad_get_allowed_caps (dts->srcpad);
+    if (caps && gst_caps_get_size (caps) > 0) {
+      GstCaps *copy = gst_caps_copy_nth (caps, 0);
+      GstStructure *structure = gst_caps_get_structure (copy, 0);
+      gint channels;
+      const int dts_channels[6] = {
+        DCA_MONO,
+        DCA_STEREO,
+        DCA_STEREO | DCA_LFE,
+        DCA_2F2R,
+        DCA_2F2R | DCA_LFE,
+        DCA_3F2R | DCA_LFE,
+      };
+
+      /* Prefer the original number of channels, but fixate to something 
+       * preferred (first in the caps) downstream if possible.
+       */
+      gst_structure_fixate_field_nearest_int (structure, "channels",
+          flags ? gst_dtsdec_channels (flags, NULL) : 6);
+      gst_structure_get_int (structure, "channels", &channels);
+      if (channels <= 6)
+        dts->request_channels = dts_channels[channels - 1];
+      else
+        dts->request_channels = dts_channels[5];
+
+      gst_caps_unref (copy);
+    } else if (flags) {
+      dts->request_channels = dts->stream_channels;
+    } else {
+      dts->request_channels = DCA_3F2R | DCA_LFE;
+    }
+
+    if (caps)
+      gst_caps_unref (caps);
+  }
+
   /* process */
-  flags = dts->request_channels | DTS_ADJUST_LEVEL;
+  flags = dts->request_channels | DCA_ADJUST_LEVEL;
   dts->level = 1;
 
-  if (dts_frame (dts->state, data, &flags, &dts->level, dts->bias)) {
+  if (dca_frame (dts->state, data, &flags, &dts->level, dts->bias)) {
     GST_WARNING ("dts_frame error");
     return GST_FLOW_OK;
   }
 
-  channels = flags & (DTS_CHANNEL_MASK | DTS_LFE);
+  channels = flags & (DCA_CHANNEL_MASK | DCA_LFE);
 
   if (dts->using_channels != channels) {
     need_renegotiation = TRUE;
@@ -418,18 +513,18 @@ gst_dtsdec_handle_frame (GstDtsDec * dts, guint8 * data,
   }
 
   if (dts->dynamic_range_compression == FALSE) {
-    dts_dynrng (dts->state, NULL, NULL);
+    dca_dynrng (dts->state, NULL, NULL);
   }
 
   /* handle decoded data, one block is 256 samples */
-  num_blocks = dts_blocks_num (dts->state);
+  num_blocks = dca_blocks_num (dts->state);
   for (i = 0; i < num_blocks; i++) {
-    if (dts_block (dts->state)) {
+    if (dca_block (dts->state)) {
       GST_WARNING ("dts_block error %d", i);
       continue;
     }
 
-    samples = dts_samples (dts->state);
+    samples = dca_samples (dts->state);
     num_c = gst_dtsdec_channels (dts->using_channels, NULL);
 
     result = gst_pad_alloc_buffer_and_set_caps (dts->srcpad, 0,
@@ -463,7 +558,7 @@ gst_dtsdec_handle_frame (GstDtsDec * dts, guint8 * data,
 }
 
 static GstFlowReturn
-gst_dtsdec_chain (GstPad * pad, GstBuffer * buf)
+gst_dtsdec_chain_raw (GstPad * pad, GstBuffer * buf)
 {
   GstDtsDec *dts;
   guint8 *data;
@@ -471,7 +566,7 @@ gst_dtsdec_chain (GstPad * pad, GstBuffer * buf)
   gint length, flags, sample_rate, bit_rate, frame_length;
   GstFlowReturn result = GST_FLOW_OK;
 
-  dts = GST_DTSDEC (gst_pad_get_parent (pad));
+  dts = GST_DTSDEC (GST_PAD_PARENT (pad));
 
   if (dts->cache) {
     buf = gst_buffer_join (dts->cache, buf);
@@ -482,7 +577,7 @@ gst_dtsdec_chain (GstPad * pad, GstBuffer * buf)
   size = GST_BUFFER_SIZE (buf);
   length = 0;
   while (size >= 7) {
-    length = dts_syncinfo (dts->state, data, &flags,
+    length = dca_syncinfo (dts->state, data, &flags,
         &sample_rate, &bit_rate, &frame_length);
     if (length == 0) {
       /* shift window to re-find sync */
@@ -514,11 +609,84 @@ gst_dtsdec_chain (GstPad * pad, GstBuffer * buf)
   }
 
   gst_buffer_unref (buf);
-  gst_object_unref (dts);
 
   return result;
 }
 
+
+static GstFlowReturn
+gst_dtsdec_chain (GstPad * pad, GstBuffer * buf)
+{
+  GstFlowReturn res = GST_FLOW_OK;
+  GstDtsDec *dts = GST_DTSDEC (GST_PAD_PARENT (pad));
+  gint first_access;
+
+  if (dts->dvdmode) {
+    gint size = GST_BUFFER_SIZE (buf);
+    guint8 *data = GST_BUFFER_DATA (buf);
+    gint offset, len;
+    GstBuffer *subbuf;
+
+    if (size < 2)
+      goto not_enough_data;
+
+    first_access = (data[0] << 8) | data[1];
+
+    /* Skip the first_access header */
+    offset = 2;
+
+    if (first_access > 1) {
+      /* Length of data before first_access */
+      len = first_access - 1;
+
+      if (len <= 0 || offset + len > size)
+        goto bad_first_access_parameter;
+
+      subbuf = gst_buffer_create_sub (buf, offset, len);
+      GST_BUFFER_TIMESTAMP (subbuf) = GST_CLOCK_TIME_NONE;
+      res = gst_dtsdec_chain_raw (pad, subbuf);
+      if (res != GST_FLOW_OK)
+        goto done;
+
+      offset += len;
+      len = size - offset;
+
+      if (len > 0) {
+        subbuf = gst_buffer_create_sub (buf, offset, len);
+        GST_BUFFER_TIMESTAMP (subbuf) = GST_BUFFER_TIMESTAMP (buf);
+
+        res = gst_dtsdec_chain_raw (pad, subbuf);
+      }
+    } else {
+      /* first_access = 0 or 1, so if there's a timestamp it applies
+       * to the first byte */
+      subbuf = gst_buffer_create_sub (buf, offset, size - offset);
+      GST_BUFFER_TIMESTAMP (subbuf) = GST_BUFFER_TIMESTAMP (buf);
+      res = gst_dtsdec_chain_raw (pad, subbuf);
+    }
+  } else {
+    res = gst_dtsdec_chain_raw (pad, buf);
+  }
+
+done:
+  return res;
+
+/* ERRORS */
+not_enough_data:
+  {
+    GST_ELEMENT_ERROR (GST_ELEMENT (dts), STREAM, DECODE, (NULL),
+        ("Insufficient data in buffer. Can't determine first_acess"));
+    return GST_FLOW_ERROR;
+  }
+bad_first_access_parameter:
+  {
+    GST_ELEMENT_ERROR (GST_ELEMENT (dts), STREAM, DECODE, (NULL),
+        ("Bad first_access parameter (%d) in buffer", first_access));
+    return GST_FLOW_ERROR;
+  }
+
+}
+
 static GstStateChangeReturn
 gst_dtsdec_change_state (GstElement * element, GstStateChange transition)
 {
@@ -530,16 +698,16 @@ gst_dtsdec_change_state (GstElement * element, GstStateChange transition)
       GstDtsDecClass *klass;
 
       klass = GST_DTSDEC_CLASS (G_OBJECT_GET_CLASS (dts));
-      dts->state = dts_init (klass->dts_cpuflags);
+      dts->state = dca_init (klass->dts_cpuflags);
       break;
     }
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      dts->samples = dts_samples (dts->state);
+      dts->samples = dca_samples (dts->state);
       dts->bit_rate = -1;
       dts->sample_rate = -1;
       dts->stream_channels = 0;
       /* FIXME force stereo for now */
-      dts->request_channels = DTS_STEREO;
+      dts->request_channels = DCA_CHANNEL;
       dts->using_channels = 0;
       dts->level = 1;
       dts->bias = 0;
@@ -564,7 +732,7 @@ gst_dtsdec_change_state (GstElement * element, GstStateChange transition)
       }
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
-      dts_free (dts->state);
+      dca_free (dts->state);
       dts->state = NULL;
       break;
     default:
