@@ -37,6 +37,12 @@
 GST_DEBUG_CATEGORY_STATIC (gst_jasper_dec_debug);
 #define GST_CAT_DEFAULT gst_jasper_dec_debug
 
+static const GstElementDetails plugin_details =
+GST_ELEMENT_DETAILS ("Jasper JPEG2000 image decoder",
+    "Codec/Decoder/Image",
+    "Decodes JPEG2000 encoded images using jasper",
+    "Mark Nauwelaerts <mnauw@users.sf.net>");
+
 enum
 {
   ARG_0,
@@ -65,9 +71,19 @@ static GstStaticPadTemplate gst_jasper_dec_src_template =
     GST_STATIC_CAPS (GST_VIDEO_CAPS_RGB "; " GST_VIDEO_CAPS_BGR "; "
         GST_VIDEO_CAPS_RGBx "; " GST_VIDEO_CAPS_xRGB "; "
         GST_VIDEO_CAPS_BGRx "; " GST_VIDEO_CAPS_xBGR "; "
-        GST_VIDEO_CAPS_YUV ("{ I420, YV12, YUY2, UYVY, Y41B, Y42B, v308 }"))
+        GST_VIDEO_CAPS_YUV ("{ I420, YV12, YUY2, UYVY, Y41B, Y42B }"))
     );
 
+static void gst_jasper_dec_base_init (gpointer g_class);
+static void gst_jasper_dec_class_init (GstJasperDecClass * klass);
+static void gst_jasper_dec_init (GstJasperDec * filter,
+    GstJasperDecClass * klass);
+
+static void gst_jasper_dec_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec);
+static void gst_jasper_dec_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec);
+
 static void gst_jasper_dec_reset (GstJasperDec * dec);
 static GstStateChangeReturn gst_jasper_dec_change_state (GstElement * element,
     GstStateChange transition);
@@ -95,27 +111,29 @@ gst_jasper_dec_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_jasper_dec_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_jasper_dec_sink_template);
-  gst_element_class_set_details_simple (element_class,
-      "Jasper JPEG2000 image decoder", "Codec/Decoder/Image",
-      "Decodes JPEG2000 encoded images using jasper",
-      "Mark Nauwelaerts <mnauw@users.sf.net>");
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_jasper_dec_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_jasper_dec_sink_template));
+  gst_element_class_set_details (element_class, &plugin_details);
 }
 
 /* initialize the plugin's class */
 static void
 gst_jasper_dec_class_init (GstJasperDecClass * klass)
 {
+  GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
 
+  gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
   GST_DEBUG_CATEGORY_INIT (gst_jasper_dec_debug, "jp2kdec", 0,
       "Jasper JPEG2000 decoder");
 
+  gobject_class->set_property = gst_jasper_dec_set_property;
+  gobject_class->get_property = gst_jasper_dec_get_property;
+
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_jasper_dec_change_state);
 }
@@ -208,7 +226,7 @@ gst_jasper_dec_sink_setcaps (GstPad * pad, GstCaps * caps)
         break;
     }
 
-    dec->fmt = jas_image_strtofmt ((char *) "jpc");
+    dec->fmt = jas_image_strtofmt ("jpc");
     /* strip the j2c box stuff it is embedded in */
     if (!strcmp (mimetype, "image/x-jpc"))
       dec->strip = 0;
@@ -221,7 +239,7 @@ gst_jasper_dec_sink_setcaps (GstPad * pad, GstCaps * caps)
       gst_buffer_ref (dec->codec_data);
     }
   } else if (!strcmp (mimetype, "image/jp2"))
-    dec->fmt = jas_image_strtofmt ((char *) "jp2");
+    dec->fmt = jas_image_strtofmt ("jp2");
 
   if (dec->fmt < 0)
     goto refuse_caps;
@@ -246,13 +264,12 @@ refuse_caps:
   }
 }
 
-static GstFlowReturn
+static gboolean
 gst_jasper_dec_negotiate (GstJasperDec * dec, jas_image_t * image)
 {
-  GstFlowReturn flow_ret = GST_FLOW_OK;
   gint width, height, channels;
   gint i, j;
-  gboolean negotiate = FALSE;
+  gboolean negotiate = FALSE, ret = TRUE;
   jas_clrspc_t clrspc;
   GstCaps *allowed_caps, *caps;
 
@@ -307,7 +324,7 @@ gst_jasper_dec_negotiate (GstJasperDec * dec, jas_image_t * image)
     goto done;
 
   /* clear and refresh to new state */
-  flow_ret = GST_FLOW_NOT_NEGOTIATED;
+  ret = FALSE;
   dec->format = GST_VIDEO_FORMAT_UNKNOWN;
   dec->width = width;
   dec->height = height;
@@ -410,30 +427,24 @@ gst_jasper_dec_negotiate (GstJasperDec * dec, jas_image_t * image)
 
     GST_DEBUG_OBJECT (dec, "Set format to %d, size to %dx%d", dec->format,
         dec->width, dec->height);
-
-    if (!gst_pad_set_caps (dec->srcpad, caps))
-      flow_ret = GST_FLOW_NOT_NEGOTIATED;
-    else
-      flow_ret = GST_FLOW_OK;
-
+    ret = gst_pad_set_caps (dec->srcpad, caps);
     gst_caps_unref (caps);
   }
 
 done:
-  return flow_ret;
+  return ret;
 
   /* ERRORS */
 fail_image:
   {
     GST_DEBUG_OBJECT (dec, "Failed to process decoded image.");
-    flow_ret = GST_FLOW_NOT_NEGOTIATED;
+    ret = FALSE;
     goto done;
   }
 not_supported:
   {
     GST_DEBUG_OBJECT (dec, "Decoded image has unsupported colour space.");
-    GST_ELEMENT_ERROR (dec, STREAM, DECODE, (NULL), ("Unsupported colorspace"));
-    flow_ret = GST_FLOW_ERROR;
+    ret = FALSE;
     goto done;
   }
 }
@@ -454,11 +465,10 @@ gst_jasper_dec_get_picture (GstJasperDec * dec, guint8 * data,
   if (!(stream = jas_stream_memopen ((gpointer) data, size)))
     goto fail_stream;
 
-  if (!(image = jas_image_decode (stream, dec->fmt, (char *) "")))
+  if (!(image = jas_image_decode (stream, dec->fmt, "")))
     goto fail_decode;
 
-  ret = gst_jasper_dec_negotiate (dec, image);
-  if (ret != GST_FLOW_OK)
+  if (!gst_jasper_dec_negotiate (dec, image))
     goto fail_negotiate;
 
   ret = gst_pad_alloc_buffer_and_set_caps (dec->srcpad,
@@ -533,8 +543,8 @@ fail:
     if (*outbuf)
       gst_buffer_unref (*outbuf);
     *outbuf = NULL;
-    GST_ELEMENT_WARNING (dec, STREAM, DECODE, (NULL), (NULL));
-    ret = GST_FLOW_OK;
+    GST_ELEMENT_ERROR (dec, STREAM, DECODE, (NULL), (NULL));
+    ret = GST_FLOW_ERROR;
     goto done;
   }
 no_buffer:
@@ -546,6 +556,7 @@ no_buffer:
 fail_negotiate:
   {
     GST_DEBUG_OBJECT (dec, "Failed to determine output caps.");
+    ret = GST_FLOW_NOT_NEGOTIATED;
     goto done;
   }
 }
@@ -809,6 +820,36 @@ invalid_bytes_segment:
   }
 }
 
+static void
+gst_jasper_dec_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec)
+{
+  GstJasperDec *filter;
+
+  filter = GST_JASPER_DEC (object);
+
+  switch (prop_id) {
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
+static void
+gst_jasper_dec_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec)
+{
+  GstJasperDec *filter;
+
+  filter = GST_JASPER_DEC (object);
+
+  switch (prop_id) {
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
 static GstStateChangeReturn
 gst_jasper_dec_change_state (GstElement * element, GstStateChange transition)
 {
