@@ -78,9 +78,6 @@
  * includes
  */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
 #include <string.h>
 
 #include "metadataparsejpeg.h"
@@ -93,9 +90,6 @@
 #include "metadataiptc.h"
 #endif
 
-GST_DEBUG_CATEGORY_EXTERN (gst_metadata_demux_debug);
-#define GST_CAT_DEFAULT gst_metadata_demux_debug
-
 /*
  * defines and macros
  */
