@@ -43,13 +43,6 @@
 #include "config.h"
 #endif
 
-/* Required to not get an undefined warning
- * https://bugzilla.gnome.org/show_bug.cgi?id=613795
- */
-#ifndef WORDS_BIGENDIAN
-#define WORDS_BIGENDIAN 0
-#endif
-
 #include <stdafx.h>
 #include <libmodplug/sndfile.h>
 
@@ -96,19 +89,12 @@ static GstStaticPadTemplate modplug_src_template_factory =
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/x-raw-int,"
-        " endianness = (int) BYTE_ORDER,"
-        " signed = (boolean) true,"
-        " width = (int) 32,"
-        " depth = (int) 32,"
-        " rate = (int) { 8000, 11025, 22050, 44100 },"
-        " channels = (int) [ 1, 2 ]; "
-        "audio/x-raw-int,"
         " endianness = (int) BYTE_ORDER,"
         " signed = (boolean) true,"
         " width = (int) 16,"
         " depth = (int) 16,"
         " rate = (int) { 8000, 11025, 22050, 44100 },"
-        " channels = (int) [ 1, 2 ]; "
+        " channels = (int) 2; "
         "audio/x-raw-int,"
         " endianness = (int) BYTE_ORDER,"
         " signed = (boolean) false,"
@@ -149,10 +135,10 @@ gst_modplug_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &modplug_sink_template_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &modplug_src_template_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&modplug_sink_template_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&modplug_src_template_factory));
 
   gst_element_class_set_details_simple (element_class, "ModPlug",
       "Codec/Decoder/Audio", "Module decoder based on modplug engine",
@@ -176,62 +162,52 @@ gst_modplug_class_init (GstModPlugClass * klass)
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SONGNAME,
       g_param_spec_string ("songname", "Songname", "The song name",
-          NULL, (GParamFlags) (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS)));
+          NULL, G_PARAM_READABLE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_REVERB,
       g_param_spec_boolean ("reverb", "reverb", "Reverb",
-          DEFAULT_REVERB,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
+          DEFAULT_REVERB, (GParamFlags) G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_REVERB_DEPTH,
       g_param_spec_int ("reverb-depth", "reverb depth", "Reverb depth",
-          0, 100, DEFAULT_REVERB_DEPTH,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
+          0, 100, DEFAULT_REVERB_DEPTH, (GParamFlags) G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_REVERB_DELAY,
       g_param_spec_int ("reverb-delay", "reverb delay", "Reverb delay",
-          0, 200, DEFAULT_REVERB_DELAY,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
+          0, 200, DEFAULT_REVERB_DELAY, (GParamFlags) G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MEGABASS,
       g_param_spec_boolean ("megabass", "megabass", "Megabass",
-          DEFAULT_MEGABASS,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
+          DEFAULT_MEGABASS, (GParamFlags) G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MEGABASS_AMOUNT,
       g_param_spec_int ("megabass-amount", "megabass amount", "Megabass amount",
-          0, 100, DEFAULT_MEGABASS_AMOUNT,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
+          0, 100, DEFAULT_MEGABASS_AMOUNT, (GParamFlags) G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MEGABASS_RANGE,
       g_param_spec_int ("megabass-range", "megabass range", "Megabass range",
-          0, 100, DEFAULT_MEGABASS_RANGE,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
+          0, 100, DEFAULT_MEGABASS_RANGE, (GParamFlags) G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SURROUND,
       g_param_spec_boolean ("surround", "surround", "Surround",
-          DEFAULT_SURROUND,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
+          DEFAULT_SURROUND, (GParamFlags) G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SURROUND_DEPTH,
       g_param_spec_int ("surround-depth", "surround depth", "Surround depth",
-          0, 100, DEFAULT_SURROUND_DEPTH,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
+          0, 100, DEFAULT_SURROUND_DEPTH, (GParamFlags) G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SURROUND_DELAY,
       g_param_spec_int ("surround-delay", "surround delay", "Surround delay",
-          0, 40, DEFAULT_SURROUND_DELAY,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
+          0, 40, DEFAULT_SURROUND_DELAY, (GParamFlags) G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_OVERSAMP,
       g_param_spec_boolean ("oversamp", "oversamp", "oversamp",
-          DEFAULT_OVERSAMP,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
+          DEFAULT_OVERSAMP, (GParamFlags) G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_NOISE_REDUCTION,
       g_param_spec_boolean ("noise-reduction", "noise reduction",
           "noise reduction", DEFAULT_NOISE_REDUCTION,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
+          (GParamFlags) G_PARAM_READWRITE));
 
   gstelement_class->change_state = gst_modplug_change_state;
 }
@@ -272,7 +248,7 @@ gst_modplug_init (GstModPlug * modplug, GstModPlugClass * klass)
   modplug->oversamp = DEFAULT_OVERSAMP;
   modplug->noise_reduction = DEFAULT_NOISE_REDUCTION;
 
-  modplug->bits = 16;
+  modplug->_16bit = TRUE;
   modplug->channel = 2;
   modplug->frequency = 44100;
 }
@@ -370,19 +346,15 @@ gst_modplug_src_event (GstPad * pad, GstEvent * event)
       GstSeekType cur_type, stop_type;
       gboolean flush;
       gint64 cur, stop;
-/* FIXME timestamp is set but not used */
-#if 0
       guint64 timestamp;
-#endif
 
       if (modplug->frequency == 0) {
         GST_DEBUG_OBJECT (modplug, "no song loaded yet");
         break;
       }
-#if 0
+
       timestamp = gst_util_uint64_scale_int (modplug->offset, GST_SECOND,
           modplug->frequency);
-#endif
 
       gst_event_parse_seek (event, &rate, &format, &flags,
           &cur_type, &cur, &stop_type, &stop);
@@ -465,10 +437,10 @@ gst_modplug_fixate (GstPad * pad, GstCaps * caps)
   GstStructure *structure;
 
   structure = gst_caps_get_structure (caps, 0);
-  if (!gst_structure_fixate_field_nearest_int (structure, "rate", 44100))
-    GST_WARNING_OBJECT (pad, "Failed to fixate rate to 44100");
-  if (!gst_structure_fixate_field_nearest_int (structure, "channels", 2))
-    GST_WARNING_OBJECT (pad, "Failed to fixate number of channels to stereo");
+  if (gst_structure_fixate_field_nearest_int (structure, "rate", 44100))
+    return;
+  if (gst_structure_fixate_field_nearest_int (structure, "channels", 2))
+    return;
 }
 
 static gboolean
@@ -476,6 +448,7 @@ gst_modplug_load_song (GstModPlug * modplug)
 {
   GstCaps *newcaps, *othercaps;
   GstStructure *structure;
+  gint depth;
 
   GST_DEBUG_OBJECT (modplug, "Setting caps");
 
@@ -494,20 +467,24 @@ gst_modplug_load_song (GstModPlug * modplug)
 
   /* set up modplug to output the negotiated format */
   structure = gst_caps_get_structure (newcaps, 0);
-  gst_structure_get_int (structure, "depth", &modplug->bits);
+  gst_structure_get_int (structure, "depth", &depth);
+  modplug->_16bit = (depth == 16);
   gst_structure_get_int (structure, "channels", &modplug->channel);
   gst_structure_get_int (structure, "rate", &modplug->frequency);
 
   modplug->read_samples = 1152;
-  modplug->read_bytes =
-      modplug->read_samples * modplug->channel * modplug->bits / 8;
+  modplug->read_bytes = modplug->read_samples * modplug->channel * depth / 8;
 
   GST_DEBUG_OBJECT (modplug, "Loading song");
 
   modplug->mSoundFile = new CSoundFile;
 
-  modplug->mSoundFile->SetWaveConfig (modplug->frequency, modplug->bits,
-      modplug->channel);
+  if (modplug->_16bit)
+    modplug->mSoundFile->SetWaveConfig (modplug->frequency, 16,
+        modplug->channel);
+  else
+    modplug->mSoundFile->SetWaveConfig (modplug->frequency, 8,
+        modplug->channel);
 
   modplug->mSoundFile->SetWaveConfigEx (modplug->surround, !modplug->oversamp,
       modplug->reverb, true, modplug->megabass, modplug->noise_reduction, true);
