@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) 2008-2009 Jan Schmidt <thaytan@noraisin.net>
+ * Copyright (C) 2008 Jan Schmidt <thaytan@noraisin.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,13 +21,11 @@
 #include <config.h>
 #endif
 
-#include <stdio.h>
 #include <string.h>
 
 #include <gst/gst.h>
 #include <gst/gst-i18n-plugin.h>
 #include <gst/interfaces/navigation.h>
-#include <gst/video/video.h>
 
 #include "resindvdsrc.h"
 
@@ -36,7 +34,6 @@ GST_DEBUG_CATEGORY_STATIC (rsndvdsrc_debug);
 
 #define DEFAULT_DEVICE "/dev/dvd"
 #define DEFAULT_FASTSTART TRUE
-#define DEFAULT_LANG "en"
 
 #define GST_FLOW_WOULD_BLOCK GST_FLOW_CUSTOM_SUCCESS
 
@@ -62,7 +59,7 @@ typedef enum
   RSN_BTN_LEFT = 0x01,
   RSN_BTN_RIGHT = 0x02,
   RSN_BTN_UP = 0x04,
-  RSN_BTN_DOWN = 0x08
+  RSN_BTN_DOWN = 0x04
 } RsnBtnMask;
 
 enum
@@ -99,8 +96,8 @@ static GstFormat chapter_format;
 
 static void rsn_dvdsrc_register_extra (GType rsn_dvdsrc_type);
 
-GST_BOILERPLATE_FULL (resinDvdSrc, rsn_dvdsrc, GstBaseSrc,
-    GST_TYPE_BASE_SRC, rsn_dvdsrc_register_extra);
+GST_BOILERPLATE_FULL (resinDvdSrc, rsn_dvdsrc, RsnPushSrc,
+    RSN_TYPE_PUSH_SRC, rsn_dvdsrc_register_extra);
 
 static gboolean read_vts_info (resinDvdSrc * src);
 
@@ -111,15 +108,15 @@ static void rsn_dvdsrc_get_property (GObject * object, guint prop_id,
 
 static void rsn_dvdsrc_finalize (GObject * object);
 
-static gboolean rsn_dvdsrc_start (GstBaseSrc * bsrc);
-static gboolean rsn_dvdsrc_stop (GstBaseSrc * bsrc);
-static gboolean rsn_dvdsrc_unlock (GstBaseSrc * bsrc);
-static gboolean rsn_dvdsrc_unlock_stop (GstBaseSrc * bsrc);
+static gboolean rsn_dvdsrc_start (RsnBaseSrc * bsrc);
+static gboolean rsn_dvdsrc_stop (RsnBaseSrc * bsrc);
+static gboolean rsn_dvdsrc_unlock (RsnBaseSrc * bsrc);
+static gboolean rsn_dvdsrc_unlock_stop (RsnBaseSrc * bsrc);
 
-static gboolean rsn_dvdsrc_is_seekable (GstBaseSrc * bsrc);
-static gboolean rsn_dvdsrc_prepare_seek (GstBaseSrc * bsrc, GstEvent * event,
+static gboolean rsn_dvdsrc_is_seekable (RsnBaseSrc * bsrc);
+static gboolean rsn_dvdsrc_prepare_seek (RsnBaseSrc * bsrc, GstEvent * event,
     GstSegment * segment);
-static gboolean rsn_dvdsrc_do_seek (GstBaseSrc * bsrc, GstSegment * segment);
+static gboolean rsn_dvdsrc_do_seek (RsnBaseSrc * bsrc, GstSegment * segment);
 static GstStateChangeReturn
 rsn_dvdsrc_change_state (GstElement * element, GstStateChange transition);
 
@@ -141,11 +138,9 @@ static void rsn_dvdsrc_check_nav_blocks (resinDvdSrc * src);
 static void rsn_dvdsrc_schedule_nav_cb (resinDvdSrc * src,
     RsnDvdPendingNav * next_nav);
 
-static gboolean rsn_dvdsrc_check_get_range (GstBaseSrc * src);
-static GstFlowReturn rsn_dvdsrc_create (GstBaseSrc * bsrc, guint64 offset,
-    guint length, GstBuffer ** buf);
-static gboolean rsn_dvdsrc_src_event (GstBaseSrc * basesrc, GstEvent * event);
-static gboolean rsn_dvdsrc_src_query (GstBaseSrc * basesrc, GstQuery * query);
+static GstFlowReturn rsn_dvdsrc_create (RsnPushSrc * psrc, GstBuffer ** buf);
+static gboolean rsn_dvdsrc_src_event (RsnBaseSrc * basesrc, GstEvent * event);
+static gboolean rsn_dvdsrc_src_query (RsnBaseSrc * basesrc, GstQuery * query);
 
 static GstClockTime ifotime_to_gsttime (dvd_time_t * ifo_time);
 static void rsn_dvdsrc_send_commands_changed (resinDvdSrc * src);
@@ -190,12 +185,17 @@ rsn_dvdsrc_register_extra (GType rsn_dvdsrc_type)
 static void
 rsn_dvdsrc_base_init (gpointer gclass)
 {
-
+  static GstElementDetails element_details = {
+    "Resin DVD Src",
+    "Source/DVD",
+    "DVD source element",
+    "Jan Schmidt <thaytan@noraisin.net>"
+  };
   GstElementClass *element_class = GST_ELEMENT_CLASS (gclass);
 
-  gst_element_class_add_static_pad_template (element_class, &src_factory);
-  gst_element_class_set_details_simple (element_class, "Resin DVD Src",
-      "Source/DVD", "DVD source element", "Jan Schmidt <thaytan@noraisin.net>");
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_factory));
+  gst_element_class_set_details (element_class, &element_details);
 }
 
 static void
@@ -203,11 +203,13 @@ rsn_dvdsrc_class_init (resinDvdSrcClass * klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
-  GstBaseSrcClass *gstbasesrc_class;
+  RsnBaseSrcClass *gstbasesrc_class;
+  RsnPushSrcClass *gstpush_src_class;
 
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
   gstbasesrc_class = GST_BASE_SRC_CLASS (klass);
+  gstpush_src_class = GST_PUSH_SRC_CLASS (klass);
 
   gobject_class->finalize = rsn_dvdsrc_finalize;
   gobject_class->set_property = rsn_dvdsrc_set_property;
@@ -226,18 +228,16 @@ rsn_dvdsrc_class_init (resinDvdSrcClass * klass)
       GST_DEBUG_FUNCPTR (rsn_dvdsrc_prepare_seek);
   gstbasesrc_class->do_seek = GST_DEBUG_FUNCPTR (rsn_dvdsrc_do_seek);
 
-  gstbasesrc_class->check_get_range =
-      GST_DEBUG_FUNCPTR (rsn_dvdsrc_check_get_range);
-  gstbasesrc_class->create = GST_DEBUG_FUNCPTR (rsn_dvdsrc_create);
+  gstpush_src_class->create = GST_DEBUG_FUNCPTR (rsn_dvdsrc_create);
 
   g_object_class_install_property (gobject_class, ARG_DEVICE,
       g_param_spec_string ("device", "Device", "DVD device location",
-          NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          NULL, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, ARG_FASTSTART,
       g_param_spec_boolean ("fast-start", "Fast start",
           "Skip straight to the DVD menu on start", DEFAULT_FASTSTART,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
 }
 
 static void
@@ -257,7 +257,7 @@ rsn_dvdsrc_init (resinDvdSrc * rsndvdsrc, resinDvdSrcClass * gclass)
   rsndvdsrc->branching = FALSE;
   rsndvdsrc->still_cond = g_cond_new ();
 
-  gst_base_src_set_format (GST_BASE_SRC (rsndvdsrc), GST_FORMAT_TIME);
+  rsn_base_src_set_format (GST_BASE_SRC (rsndvdsrc), GST_FORMAT_TIME);
 }
 
 static void
@@ -268,7 +268,6 @@ rsn_dvdsrc_finalize (GObject * object)
   g_mutex_free (src->dvd_lock);
   g_mutex_free (src->branch_lock);
   g_cond_free (src->still_cond);
-  g_free (src->device);
 
   gst_buffer_replace (&src->alloc_buf, NULL);
   gst_buffer_replace (&src->next_buf, NULL);
@@ -277,7 +276,7 @@ rsn_dvdsrc_finalize (GObject * object)
 }
 
 static gboolean
-rsn_dvdsrc_unlock (GstBaseSrc * bsrc)
+rsn_dvdsrc_unlock (RsnBaseSrc * bsrc)
 {
   resinDvdSrc *src = RESINDVDSRC (bsrc);
 
@@ -290,7 +289,7 @@ rsn_dvdsrc_unlock (GstBaseSrc * bsrc)
 }
 
 static gboolean
-rsn_dvdsrc_unlock_stop (GstBaseSrc * bsrc)
+rsn_dvdsrc_unlock_stop (RsnBaseSrc * bsrc)
 {
   resinDvdSrc *src = RESINDVDSRC (bsrc);
 
@@ -352,11 +351,9 @@ rsn_dvdsrc_get_property (GObject * object, guint prop_id,
 }
 
 static gboolean
-rsn_dvdsrc_start (GstBaseSrc * bsrc)
+rsn_dvdsrc_start (RsnBaseSrc * bsrc)
 {
   resinDvdSrc *src = RESINDVDSRC (bsrc);
-  const gchar *const *langs, *const *cur;
-  gchar lang[8];
 
   g_mutex_lock (src->dvd_lock);
   if (!read_vts_info (src)) {
@@ -377,21 +374,6 @@ rsn_dvdsrc_start (GstBaseSrc * bsrc)
     goto fail;
   }
 
-  /* Attempt to set DVD menu, audio and spu languages */
-  langs = g_get_language_names ();
-  strncpy (lang, DEFAULT_LANG, 8);
-  for (cur = langs; *cur != NULL; cur++) {
-    /* Look for a 2 char iso-639 lang */
-    if (strlen (*cur) == 2) {
-      strncpy (lang, *cur, 8);
-      break;
-    }
-  }
-  /* Set the user's preferred language */
-  dvdnav_menu_language_select (src->dvdnav, lang);
-  dvdnav_audio_language_select (src->dvdnav, lang);
-  dvdnav_spu_language_select (src->dvdnav, lang);
-
   if (src->faststart) {
     if (dvdnav_title_play (src->dvdnav, 1) != DVDNAV_STATUS_OK ||
         (dvdnav_menu_call (src->dvdnav, DVD_MENU_Title) != DVDNAV_STATUS_OK &&
@@ -409,7 +391,6 @@ rsn_dvdsrc_start (GstBaseSrc * bsrc)
   src->branching = FALSE;
   src->discont = TRUE;
   src->need_segment = TRUE;
-  src->need_tag_update = TRUE;
 
   src->cur_position = GST_CLOCK_TIME_NONE;
   src->pgc_duration = GST_CLOCK_TIME_NONE;
@@ -527,7 +508,7 @@ get_vts_attr (resinDvdSrc * src, gint n)
 }
 
 static gboolean
-rsn_dvdsrc_stop (GstBaseSrc * bsrc)
+rsn_dvdsrc_stop (RsnBaseSrc * bsrc)
 {
   resinDvdSrc *src = RESINDVDSRC (bsrc);
   gboolean ret = TRUE;
@@ -613,6 +594,7 @@ rsn_dvdsrc_do_still (resinDvdSrc * src, int duration)
   GstEvent *still_event;
   GstEvent *hl_event;
   gboolean cmds_changed;
+  GstStructure *s;
   GstEvent *seg_event;
   GstSegment *segment = &(GST_BASE_SRC (src)->segment);
 
@@ -625,9 +607,12 @@ rsn_dvdsrc_do_still (resinDvdSrc * src, int duration)
     else
       src->still_time_remaining = GST_SECOND * duration;
 
-    /* Send a close-segment event, and a still-frame start
+    /* Send a close-segment event, and a dvd-still start
      * event, then sleep */
-    still_event = gst_video_event_new_still_frame (TRUE);
+    s = gst_structure_new ("application/x-gst-dvd",
+        "event", G_TYPE_STRING, "dvd-still",
+        "still-state", G_TYPE_BOOLEAN, TRUE, NULL);
+    still_event = gst_event_new_custom (GST_EVENT_CUSTOM_DOWNSTREAM, s);
 
     gst_segment_set_last_stop (segment, GST_FORMAT_TIME, src->cur_end_ts);
 
@@ -746,7 +731,10 @@ rsn_dvdsrc_do_still (resinDvdSrc * src, int duration)
 
     /* Tell downstream the still is over.
      * We only do this if the still isn't interrupted: */
-    still_event = gst_video_event_new_still_frame (FALSE);
+    s = gst_structure_new ("application/x-gst-dvd",
+        "event", G_TYPE_STRING, "dvd-still",
+        "still-state", G_TYPE_BOOLEAN, FALSE, NULL);
+    still_event = gst_event_new_custom (GST_EVENT_CUSTOM_DOWNSTREAM, s);
 
     /* If the segment was too short in a timed still, it may need extending */
     if (segment->last_stop < segment->start + GST_SECOND * duration)
@@ -811,8 +799,8 @@ get_current_pgc (resinDvdSrc * src)
   return pgc;
 }
 
-static GstTagList *
-update_title_info (resinDvdSrc * src, gboolean force)
+static void
+update_title_info (resinDvdSrc * src)
 {
   gint n_angles, cur_agl;
   gint title_n, part_n;
@@ -826,14 +814,14 @@ update_title_info (resinDvdSrc * src, gboolean force)
   if (dvdnav_current_title_info (src->dvdnav, &title_n,
           &part_n) != DVDNAV_STATUS_OK) {
     if (!src->in_menu)
-      return NULL;              /* Can't update now */
+      return;                   /* Can't update now */
     /* Must be in the first play sequence */
     title_n = -1;
     part_n = 0;
   }
 
   if (title_n != src->title_n || part_n != src->part_n ||
-      src->n_angles != n_angles || src->cur_angle != cur_agl || force) {
+      src->n_angles != n_angles || src->cur_angle != cur_agl) {
     gchar *title_str = NULL;
 
     src->title_n = title_n;
@@ -871,30 +859,9 @@ update_title_info (resinDvdSrc * src, gboolean force)
       gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE, GST_TAG_TITLE,
           title_str, NULL);
       g_free (title_str);
-      return tags;
+      gst_element_found_tags (GST_ELEMENT_CAST (src), tags);
     }
   }
-
-  return NULL;
-}
-
-/* we don't cache the result on purpose */
-static gboolean
-rsn_descrambler_available (void)
-{
-  GModule *module;
-  gpointer sym;
-  gsize res;
-
-  module = g_module_open ("libdvdcss", 0);
-  if (module != NULL) {
-    res = g_module_symbol (module, "dvdcss_open", &sym);
-    g_module_close (module);
-  } else {
-    res = FALSE;
-  }
-
-  return res;
 }
 
 static GstFlowReturn
@@ -906,10 +873,8 @@ rsn_dvdsrc_step (resinDvdSrc * src, gboolean have_dvd_lock)
   gint event, len;
 
   /* Allocate an output buffer if there isn't a pending one */
-  if (src->alloc_buf == NULL) {
+  if (src->alloc_buf == NULL)
     src->alloc_buf = gst_buffer_new_and_alloc (DVD_VIDEO_LB_LEN);
-    gst_buffer_set_caps (src->alloc_buf, GST_PAD_CAPS (GST_BASE_SRC_PAD (src)));
-  }
 
   data = GST_BUFFER_DATA (src->alloc_buf);
   len = DVD_VIDEO_LB_LEN;
@@ -1049,7 +1014,8 @@ rsn_dvdsrc_step (resinDvdSrc * src, gboolean have_dvd_lock)
           GST_TIME_ARGS (src->cur_position));
 
       rsn_dvdsrc_prepare_streamsinfo_event (src);
-      src->need_tag_update = TRUE;
+
+      update_title_info (src);
 
       break;
     }
@@ -1078,11 +1044,11 @@ rsn_dvdsrc_step (resinDvdSrc * src, gboolean have_dvd_lock)
       dvdnav_audio_stream_change_event_t *event =
           (dvdnav_audio_stream_change_event_t *) data;
 
-      rsn_dvdsrc_prepare_audio_stream_event (src,
-          event->logical, event->physical);
       GST_DEBUG_OBJECT (src, "  physical: %d", event->physical);
       GST_DEBUG_OBJECT (src, "  logical: %d", event->logical);
 
+      rsn_dvdsrc_prepare_audio_stream_event (src,
+          event->logical, event->physical);
       break;
     }
     case DVDNAV_SPU_STREAM_CHANGE:{
@@ -1103,8 +1069,9 @@ rsn_dvdsrc_step (resinDvdSrc * src, gboolean have_dvd_lock)
       break;
     }
     case DVDNAV_HIGHLIGHT:{
+      dvdnav_highlight_event_t *event = (dvdnav_highlight_event_t *) data;
       GST_DEBUG_OBJECT (src, "highlight change event, button %d",
-          ((dvdnav_highlight_event_t *) data)->buttonN);
+          event->buttonN);
       rsn_dvdsrc_update_highlight (src);
       break;
     }
@@ -1119,7 +1086,7 @@ rsn_dvdsrc_step (resinDvdSrc * src, gboolean have_dvd_lock)
       break;
   }
 
-  if (src->highlight_event && have_dvd_lock && src->in_playing) {
+  if (src->highlight_event && have_dvd_lock) {
     GstEvent *hl_event = src->highlight_event;
 
     src->highlight_event = NULL;
@@ -1131,35 +1098,19 @@ rsn_dvdsrc_step (resinDvdSrc * src, gboolean have_dvd_lock)
   }
 
   return ret;
-
-/* ERRORS */
 read_error:
-  {
-    if (!rsn_descrambler_available ()) {
-      GST_ELEMENT_ERROR (src, RESOURCE, READ,
-          (_("Could not read DVD. This may be because the DVD is encrypted "
-                  "and a DVD decryption library is not installed.")),
-          ("Failed to read next DVD block. Error: %s",
-              dvdnav_err_to_string (src->dvdnav)));
-    } else {
-      GST_ELEMENT_ERROR (src, RESOURCE, READ, (_("Could not read DVD.")),
-          ("Failed to read next DVD block. Error: %s",
-              dvdnav_err_to_string (src->dvdnav)));
-    }
-    return GST_FLOW_ERROR;
-  }
+  GST_ELEMENT_ERROR (src, RESOURCE, READ, (NULL),
+      ("Failed to read next DVD block. Error: %s",
+          dvdnav_err_to_string (src->dvdnav)));
+  return GST_FLOW_ERROR;
 internal_error:
-  {
-    GST_ELEMENT_ERROR (src, RESOURCE, READ, (_("Could not read DVD.")),
-        ("Internal error processing DVD commands. Error: %s",
-            dvdnav_err_to_string (src->dvdnav)));
-    return GST_FLOW_ERROR;
-  }
+  GST_ELEMENT_ERROR (src, RESOURCE, READ, (NULL),
+      ("Internal error processing DVD commands. Error: %s",
+          dvdnav_err_to_string (src->dvdnav)));
+  return GST_FLOW_ERROR;
 branching:
-  {
-    g_mutex_unlock (src->branch_lock);
-    return GST_FLOW_WRONG_STATE;
-  }
+  g_mutex_unlock (src->branch_lock);
+  return GST_FLOW_WRONG_STATE;
 }
 
 /* Send app a bus message that the available commands have changed */
@@ -1275,19 +1226,10 @@ rsn_dvdsrc_prepare_next_block (resinDvdSrc * src, gboolean have_dvd_lock)
   return ret;
 }
 
-static gboolean
-rsn_dvdsrc_check_get_range (GstBaseSrc * src)
-{
-  /* ResinDVD never operates in pull mode. There might be
-   * a reason to in the future though? */
-  return FALSE;
-}
-
 static GstFlowReturn
-rsn_dvdsrc_create (GstBaseSrc * bsrc, guint64 offset,
-    guint length, GstBuffer ** outbuf)
+rsn_dvdsrc_create (RsnPushSrc * psrc, GstBuffer ** outbuf)
 {
-  resinDvdSrc *src = RESINDVDSRC (bsrc);
+  resinDvdSrc *src = RESINDVDSRC (psrc);
   GstSegment *segment = &(GST_BASE_SRC (src)->segment);
   GstFlowReturn ret;
   GstEvent *streams_event = NULL;
@@ -1296,7 +1238,6 @@ rsn_dvdsrc_create (GstBaseSrc * bsrc, guint64 offset,
   GstEvent *audio_select_event = NULL;
   GstEvent *highlight_event = NULL;
   GstMessage *angles_msg = NULL;
-  GstTagList *tags = NULL;
   gboolean cmds_changed = FALSE;
 
   *outbuf = NULL;
@@ -1334,11 +1275,6 @@ rsn_dvdsrc_create (GstBaseSrc * bsrc, guint64 offset,
   cmds_changed = src->commands_changed;
   src->commands_changed = FALSE;
 
-  if (src->need_tag_update) {
-    tags = update_title_info (src, FALSE);
-    src->need_tag_update = FALSE;
-  }
-
   g_mutex_unlock (src->dvd_lock);
 
   /* Push in-band events now that we've dropped the dvd_lock, before
@@ -1363,35 +1299,19 @@ rsn_dvdsrc_create (GstBaseSrc * bsrc, guint64 offset,
 
   if (src->need_segment) {
     /* Seamless segment update */
-    GstClockTime position = 0;
-
-    if (src->cur_position != GST_CLOCK_TIME_NONE)
-      position += src->cur_position;
-    if (src->cur_vobu_base_ts != GST_CLOCK_TIME_NONE)
-      position += src->cur_vobu_base_ts;
-
-    GST_DEBUG_OBJECT (src,
-        "Starting seamless segment update to %" GST_TIME_FORMAT " -> %"
-        GST_TIME_FORMAT " VOBU %" GST_TIME_FORMAT " position %" GST_TIME_FORMAT,
-        GST_TIME_ARGS (src->cur_start_ts), GST_TIME_ARGS (src->cur_end_ts),
-        GST_TIME_ARGS (src->cur_vobu_base_ts), GST_TIME_ARGS (position));
-
-    gst_base_src_new_seamless_segment (GST_BASE_SRC (src),
-        src->cur_start_ts, -1, position);
+    GstEvent *seek;
 
+    seek = gst_event_new_seek (segment->rate, rsndvd_format,
+        GST_SEEK_FLAG_NONE, GST_SEEK_TYPE_NONE, -1, GST_SEEK_TYPE_NONE, -1);
+    gst_element_send_event (GST_ELEMENT (src), seek);
     src->need_segment = FALSE;
   }
 
+  g_mutex_lock (src->dvd_lock);
+
   if (src->cur_end_ts != GST_CLOCK_TIME_NONE)
     gst_segment_set_last_stop (segment, GST_FORMAT_TIME, src->cur_end_ts);
 
-  if (tags) {
-    gst_element_found_tags_for_pad (GST_ELEMENT_CAST (src),
-        GST_BASE_SRC_PAD (src), tags);
-    tags = NULL;
-  }
-  g_mutex_lock (src->dvd_lock);
-
   if (src->next_buf != NULL) {
     /* Now that we're in the new segment, we can enqueue any nav packet
      * correctly */
@@ -1410,12 +1330,8 @@ rsn_dvdsrc_create (GstBaseSrc * bsrc, guint64 offset,
     }
   }
 
-  if (src->in_playing) {
-    highlight_event = src->highlight_event;
-    src->highlight_event = NULL;
-  } else {
-    highlight_event = NULL;
-  }
+  highlight_event = src->highlight_event;
+  src->highlight_event = NULL;
 
   /* Schedule a clock callback for the any pending nav packet */
   rsn_dvdsrc_check_nav_blocks (src);
@@ -1646,11 +1562,15 @@ rsn_dvdsrc_handle_navigation_event (resinDvdSrc * src, GstEvent * event)
         gint title = 0;
         gint part = 0;
 
-        if (dvdnav_current_title_info (src->dvdnav, &title, &part)) {
-          if (title > 0 && part > 1) {
+        if (dvdnav_current_title_info (src->dvdnav, &title, &part) && title > 0
+            && part > 1) {
+          if (dvdnav_part_play (src->dvdnav, title, part - 1) ==
+              DVDNAV_STATUS_ERR)
             dvdnav_prev_pg_search (src->dvdnav);
-            nav_res = RSN_NAV_RESULT_BRANCH;
-          }
+          nav_res = RSN_NAV_RESULT_BRANCH;
+        } else {
+          dvdnav_prev_pg_search (src->dvdnav);
+          nav_res = RSN_NAV_RESULT_BRANCH;
         }
       } else if (g_str_equal (key, "period")) {
         dvdnav_next_pg_search (src->dvdnav);
@@ -1661,10 +1581,6 @@ rsn_dvdsrc_handle_navigation_event (resinDvdSrc * src, GstEvent * event)
       } else if (g_str_equal (key, "bracketright")) {
         nav_res =
             rsn_dvdsrc_do_command (src, GST_NAVIGATION_COMMAND_NEXT_ANGLE);
-      } else if (key && key[0] >= '1' && key[0] <= '8') {
-        gint new_stream = key[0] - '1';
-        GST_INFO_OBJECT (src, "Selecting audio stream %d", new_stream);
-        rsn_dvdsrc_prepare_audio_stream_event (src, new_stream, new_stream);
       }
       break;
     }
@@ -1801,7 +1717,7 @@ rsn_dvdsrc_handle_navigation_event (resinDvdSrc * src, GstEvent * event)
       }
       src->angles_changed = FALSE;
 
-      src->need_tag_update = TRUE;
+      update_title_info (src);
     }
 
     cmds_changed = src->commands_changed;
@@ -1898,7 +1814,6 @@ rsn_dvdsrc_prepare_streamsinfo_event (resinDvdSrc * src)
   audio_attr_t *a_attrs;
   subp_attr_t *s_attrs;
   gint n_audio, n_subp;
-  int8_t cur_audio;
   GstStructure *s;
   GstEvent *e;
   gint i;
@@ -1953,8 +1868,6 @@ rsn_dvdsrc_prepare_streamsinfo_event (resinDvdSrc * src)
       NULL);
 
   /* audio */
-  cur_audio = dvdnav_get_active_audio_stream (src->dvdnav);
-
   have_audio = FALSE;
   for (i = 0; i < n_audio; i++) {
     const audio_attr_t *a = a_attrs + i;
@@ -1968,23 +1881,17 @@ rsn_dvdsrc_prepare_streamsinfo_event (resinDvdSrc * src)
 
     GST_DEBUG_OBJECT (src, "mapped logical audio %d to MPEG substream %d",
         i, phys_id);
-    /* Force audio stream reselection in case format changed ... */
-    if (i == cur_audio) {
-      src->cur_audio_phys_stream = -1;
-      rsn_dvdsrc_prepare_audio_stream_event (src, i, phys_id);
-    }
-#if 0
-    /* Old test code: Only output A52 streams */
+
+#if 1
+    /* FIXME: Only output A52 streams for now, until the decoder switching
+     * is ready */
     if (a->audio_format != 0) {
       GST_DEBUG_OBJECT (src, "Ignoring non-A52 stream %d, format %d", i,
           (int) a->audio_format);
       continue;
     }
-    if (a->audio_format == 0)
-      have_audio = TRUE;
-#else
-    have_audio = TRUE;
 #endif
+    have_audio = TRUE;
 
     GST_DEBUG_OBJECT (src, "Audio stream %d is format %d, substream %d", i,
         (int) a->audio_format, phys_id);
@@ -1997,9 +1904,7 @@ rsn_dvdsrc_prepare_streamsinfo_event (resinDvdSrc * src)
     gst_structure_set (s, t, G_TYPE_INT, (int) a->audio_format, NULL);
     g_free (t);
 
-    /* Check that the language code is flagged and at least somewhat valid
-     * before putting it in the output structure */
-    if (a->lang_type && a->lang_code > 0x100) {
+    if (a->lang_type) {
       t = g_strdup_printf ("audio-%d-language", i);
       lang_code[0] = (a->lang_code >> 8) & 0xff;
       lang_code[1] = a->lang_code & 0xff;
@@ -2008,11 +1913,11 @@ rsn_dvdsrc_prepare_streamsinfo_event (resinDvdSrc * src)
 
       GST_DEBUG_OBJECT (src, "Audio stream %d is language %s", i, lang_code);
     } else
-      GST_DEBUG_OBJECT (src, "Audio stream %d - no language", i);
+      GST_DEBUG_OBJECT (src, "Audio stream %d - no language %s", i, lang_code);
   }
 
   if (have_audio == FALSE) {
-    /* Always create at least one audio stream of the required type */
+    /* Always create at least one audio stream */
     gst_structure_set (s, "audio-0-format", G_TYPE_INT, (int) 0,
         "audio-0-stream", G_TYPE_INT, (int) 0, NULL);
   }
@@ -2042,7 +1947,7 @@ rsn_dvdsrc_prepare_streamsinfo_event (resinDvdSrc * src)
     g_free (t);
 
     t = g_strdup_printf ("subpicture-%d-language", i);
-    if (u->type && u->lang_code > 0x100) {
+    if (u->type) {
       lang_code[0] = (u->lang_code >> 8) & 0xff;
       lang_code[1] = u->lang_code & 0xff;
       gst_structure_set (s, t, G_TYPE_STRING, lang_code, NULL);
@@ -2329,7 +2234,7 @@ rsn_dvdsrc_nav_clock_cb (GstClock * clock, GstClockTime time, GstClockID id,
   return TRUE;
 }
 
-/* Called with dvd_lock held. NOTE: Releases dvd_lock briefly */
+/* Called with dvd_lock held */
 static void
 rsn_dvdsrc_schedule_nav_cb (resinDvdSrc * src, RsnDvdPendingNav * next_nav)
 {
@@ -2360,10 +2265,8 @@ rsn_dvdsrc_schedule_nav_cb (resinDvdSrc * src, RsnDvdPendingNav * next_nav)
   GST_LOG_OBJECT (src, "Schedule nav pack for running TS %" GST_TIME_FORMAT,
       GST_TIME_ARGS (next_nav->running_ts));
 
-  g_mutex_unlock (src->dvd_lock);
   gst_clock_id_wait_async (src->nav_clock_id, rsn_dvdsrc_nav_clock_cb, src);
   gst_object_unref (clock);
-  g_mutex_lock (src->dvd_lock);
 }
 
 /* Called with dvd_lock held */
@@ -2389,7 +2292,7 @@ rsn_dvdsrc_check_nav_blocks (resinDvdSrc * src)
 }
 
 static gboolean
-rsn_dvdsrc_src_event (GstBaseSrc * basesrc, GstEvent * event)
+rsn_dvdsrc_src_event (RsnBaseSrc * basesrc, GstEvent * event)
 {
   resinDvdSrc *src = RESINDVDSRC (basesrc);
   gboolean res;
@@ -2465,7 +2368,7 @@ rsn_dvdsrc_change_state (GstElement * element, GstStateChange transition)
 }
 
 static gboolean
-rsn_dvdsrc_src_query (GstBaseSrc * basesrc, GstQuery * query)
+rsn_dvdsrc_src_query (RsnBaseSrc * basesrc, GstQuery * query)
 {
   resinDvdSrc *src = RESINDVDSRC (basesrc);
   gboolean res = FALSE;
@@ -2558,13 +2461,13 @@ rsn_dvdsrc_src_query (GstBaseSrc * basesrc, GstQuery * query)
 }
 
 static gboolean
-rsn_dvdsrc_is_seekable (GstBaseSrc * bsrc)
+rsn_dvdsrc_is_seekable (RsnBaseSrc * bsrc)
 {
   return TRUE;
 }
 
 static gboolean
-rsn_dvdsrc_prepare_seek (GstBaseSrc * bsrc, GstEvent * event,
+rsn_dvdsrc_prepare_seek (RsnBaseSrc * bsrc, GstEvent * event,
     GstSegment * segment)
 {
   GstSeekType cur_type, stop_type;
@@ -2593,7 +2496,7 @@ rsn_dvdsrc_prepare_seek (GstBaseSrc * bsrc, GstEvent * event,
     return TRUE;
   }
 
-  /* Let basesrc handle other formats */
+  /* Let basesrc handle other formats for now. FIXME: Implement angle */
   ret = GST_BASE_SRC_CLASS (parent_class)->prepare_seek_segment (bsrc,
       event, segment);
 
@@ -2653,9 +2556,6 @@ rsn_dvdsrc_get_sector_from_time_tmap (resinDvdSrc * src, GstClockTime ts)
 
   /* Get the time map */
   title_tmap = vts_tmapt->tmap + vts_ttn - 1;
-  if (title_tmap->tmu == 0)
-    return -1;
-
   entry = ts / (title_tmap->tmu * GST_SECOND);
   if (entry == 0)
     return 0;
@@ -2723,7 +2623,7 @@ rsn_dvdsrc_seek_to_time (resinDvdSrc * src, GstClockTime ts)
 }
 
 static gboolean
-rsn_dvdsrc_do_seek (GstBaseSrc * bsrc, GstSegment * segment)
+rsn_dvdsrc_do_seek (RsnBaseSrc * bsrc, GstSegment * segment)
 {
   resinDvdSrc *src = RESINDVDSRC (bsrc);
   gboolean ret = FALSE;
@@ -2733,10 +2633,10 @@ rsn_dvdsrc_do_seek (GstBaseSrc * bsrc, GstSegment * segment)
      * everything up and flushing, we just need to step to the next
      * data block (below) so we know our new position */
     ret = TRUE;
-    /* HACK to make initial seek work: */
     src->first_seek = FALSE;
   } else {
-    /* Handle other formats: Time, title, chapter, angle */
+    /* FIXME: Handle other formats: Time, title, chapter, angle */
+    /* HACK to make initial seek work: */
     if (segment->format == GST_FORMAT_TIME) {
       g_mutex_lock (src->dvd_lock);
       src->discont = TRUE;
@@ -2763,12 +2663,7 @@ rsn_dvdsrc_do_seek (GstBaseSrc * bsrc, GstSegment * segment)
         if (dvdnav_current_title_info (src->dvdnav, &title, &x) ==
             DVDNAV_STATUS_OK) {
           if (segment->start + 1 == x) {
-            /* if already on the first part, don't try to get before it */
-            if (segment->start == 0) {
-              dvdnav_part_play (src->dvdnav, title, 1);
-            } else {
-              dvdnav_prev_pg_search (src->dvdnav);
-            }
+            dvdnav_prev_pg_search (src->dvdnav);
             ret = TRUE;
             src->discont = TRUE;
           } else if (segment->start == x + 1) {
@@ -2822,12 +2717,8 @@ rsn_dvdsrc_do_seek (GstBaseSrc * bsrc, GstSegment * segment)
 
     GST_LOG_OBJECT (src, "Entering prepare_next_block after seek."
         " Flushing = %d", src->flushing_seek);
-    while (src->cur_start_ts == GST_CLOCK_TIME_NONE) {
-      if (rsn_dvdsrc_prepare_next_block (src, FALSE) != GST_FLOW_OK)
-        goto fail;
-      if (src->cur_start_ts == GST_CLOCK_TIME_NONE)
-        gst_buffer_replace (&src->next_buf, NULL);
-    }
+    if (rsn_dvdsrc_prepare_next_block (src, FALSE) != GST_FLOW_OK)
+      goto fail;
     GST_LOG_OBJECT (src, "prepare_next_block after seek done");
 
     segment->format = GST_FORMAT_TIME;
@@ -2857,3 +2748,14 @@ fail:
   GST_DEBUG_OBJECT (src, "Seek in format %d failed", segment->format);
   return FALSE;
 }
+
+gboolean
+rsndvdsrc_init (GstPlugin * plugin)
+{
+  gboolean res;
+
+  res = gst_element_register (plugin, "rsndvdsrc",
+      GST_RANK_NONE, RESIN_TYPE_DVDSRC);
+
+  return res;
+}
