@@ -23,7 +23,6 @@
 #include <string.h>
 
 #include <gst/gst.h>
-#include <gst/video/video.h>
 
 #include "rsnaudiomunge.h"
 
@@ -80,17 +79,22 @@ rsn_audiomunge_class_init (RsnAudioMungeClass * klass)
 {
   GObjectClass *gobject_class = (GObjectClass *) (klass);
   GstElementClass *element_class = (GstElementClass *) (klass);
+  static GstElementDetails element_details = {
+    "RsnAudioMunge",
+    "Audio/Filter",
+    "Resin DVD audio stream regulator",
+    "Jan Schmidt <thaytan@noraisin.net>"
+  };
 
   GST_DEBUG_CATEGORY_INIT (rsn_audiomunge_debug, "rsnaudiomunge",
       0, "ResinDVD audio stream regulator");
 
-  gst_element_class_add_static_pad_template (element_class, &src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
 
-  gst_element_class_set_details_simple (element_class, "RsnAudioMunge",
-      "Audio/Filter",
-      "Resin DVD audio stream regulator", "Jan Schmidt <thaytan@noraisin.net>");
+  gst_element_class_set_details (element_class, &element_details);
 
   gobject_class->set_property = rsn_audiomunge_set_property;
   gobject_class->get_property = rsn_audiomunge_get_property;
@@ -154,9 +158,9 @@ rsn_audiomunge_set_caps (GstPad * pad, GstCaps * caps)
   g_return_val_if_fail (munge != NULL, FALSE);
 
   otherpad = (pad == munge->srcpad) ? munge->sinkpad : munge->srcpad;
+  gst_object_unref (munge);
 
   ret = gst_pad_set_caps (otherpad, caps);
-  gst_object_unref (munge);
   return ret;
 }
 
@@ -196,13 +200,10 @@ rsn_audiomunge_make_audio (RsnAudioMunge * munge,
   guint buf_size;
 
   /* Just generate a 48khz stereo buffer for now */
-  /* FIXME: Adapt to the allowed formats, according to the currently
-   * plugged decoder, or at least add a source pad that accepts the
-   * caps we're outputting if the upstream decoder does not */
 #if 0
   caps =
       gst_caps_from_string
-      ("audio/x-raw-int,rate=48000,channels=2,width=16,depth=16,signed=(boolean)true,endianness=4321");
+      ("audio/x-raw-int,rate=48000,channels=2,width=16,depth=16,signed=(boolean)true,endianness=1234");
   buf_size = 4 * (48000 * fill_time / GST_SECOND);
 #else
   caps = gst_caps_from_string ("audio/x-raw-float, endianness=(int)1234,"
@@ -230,6 +231,31 @@ rsn_audiomunge_make_audio (RsnAudioMunge * munge,
   return ret;
 }
 
+static void
+rsn_audiomunge_handle_dvd_event (RsnAudioMunge * munge, GstEvent * event)
+{
+  const GstStructure *s;
+  const gchar *event_type;
+
+  s = gst_event_get_structure (event);
+  event_type = gst_structure_get_string (s, "event");
+  if (event_type == NULL)
+    return;
+
+  if (strcmp (event_type, "dvd-still") == 0) {
+    gboolean in_still;
+
+    if (!gst_structure_get_boolean (s, "still-state", &in_still))
+      return;
+
+    /* Remember the still-frame state, so we can generate a pre-roll buffer
+     * when a new-segment arrives */
+    munge->in_still = in_still;
+
+    GST_INFO_OBJECT (munge, "AUDIO MUNGE: still-state now %d", munge->in_still);
+  }
+}
+
 static gboolean
 rsn_audiomunge_sink_event (GstPad * pad, GstEvent * event)
 {
@@ -317,15 +343,10 @@ rsn_audiomunge_sink_event (GstPad * pad, GstEvent * event)
     }
     case GST_EVENT_CUSTOM_DOWNSTREAM:
     {
-      gboolean in_still;
+      const GstStructure *s = gst_event_get_structure (event);
 
-      if (gst_video_event_parse_still_frame (event, &in_still)) {
-        /* Remember the still-frame state, so we can generate a pre-roll
-         * buffer when a new-segment arrives */
-        munge->in_still = in_still;
-        GST_INFO_OBJECT (munge, "AUDIO MUNGE: still-state now %d",
-            munge->in_still);
-      }
+      if (s && gst_structure_has_name (s, "application/x-gst-dvd"))
+        rsn_audiomunge_handle_dvd_event (munge, event);
 
       ret = gst_pad_push_event (munge->srcpad, event);
       break;
@@ -335,7 +356,6 @@ rsn_audiomunge_sink_event (GstPad * pad, GstEvent * event)
       break;
   }
 
-  gst_object_unref (munge);
   return ret;
 
 newseg_wrong_format:
