@@ -51,6 +51,13 @@
 GST_DEBUG_CATEGORY_STATIC (legacyresample_debug);
 #define GST_CAT_DEFAULT legacyresample_debug
 
+/* elementfactory information */
+static const GstElementDetails gst_legacyresample_details =
+GST_ELEMENT_DETAILS ("Audio scaler",
+    "Filter/Converter/Audio",
+    "Resample audio",
+    "David Schleef <ds@schleef.org>");
+
 #define DEFAULT_FILTERLEN       16
 
 enum
@@ -135,14 +142,12 @@ gst_legacyresample_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_legacyresample_src_template);
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_legacyresample_sink_template);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_legacyresample_src_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_legacyresample_sink_template));
 
-  gst_element_class_set_details_simple (gstelement_class, "Audio scaler",
-      "Filter/Converter/Audio",
-      "Resample audio", "David Schleef <ds@schleef.org>");
+  gst_element_class_set_details (gstelement_class, &gst_legacyresample_details);
 }
 
 static void
