@@ -35,7 +35,6 @@
 #define TS_LATENCY 700
 
 #define TABLE_ID_UNSET 0xFF
-#define RUNNING_STATUS_RUNNING 4
 
 GST_DEBUG_CATEGORY_STATIC (mpegts_parse_debug);
 #define GST_CAT_DEFAULT mpegts_parse_debug
@@ -74,17 +73,14 @@ struct _MpegTSParsePad
 
   /* the return of the latest push */
   GstFlowReturn flow_return;
-
-  GstTagList *tags;
-  guint event_id;
 };
 
-static GQuark QUARK_PROGRAMS;
-static GQuark QUARK_PROGRAM_NUMBER;
-static GQuark QUARK_PID;
-static GQuark QUARK_PCR_PID;
-static GQuark QUARK_STREAMS;
-static GQuark QUARK_STREAM_TYPE;
+static GstElementDetails mpegts_parse_details =
+GST_ELEMENT_DETAILS ("MPEG transport stream parser",
+    "Codec/Parser",
+    "Parses MPEG2 transport streams",
+    "Alessandro Decina <alessandro@nnva.org>\n"
+    "Zaheer Abbas Merali <zaheerabbas at merali dot org>");
 
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -138,14 +134,8 @@ static gboolean mpegts_parse_sink_event (GstPad * pad, GstEvent * event);
 static GstStateChangeReturn mpegts_parse_change_state (GstElement * element,
     GstStateChange transition);
 static gboolean mpegts_parse_src_pad_query (GstPad * pad, GstQuery * query);
-static void _extra_init (GType type);
-static void mpegts_parse_get_tags_from_sdt (MpegTSParse * parse,
-    GstStructure * sdt_info);
-static void mpegts_parse_get_tags_from_eit (MpegTSParse * parse,
-    GstStructure * eit_info);
 
-GST_BOILERPLATE_FULL (MpegTSParse, mpegts_parse, GstElement, GST_TYPE_ELEMENT,
-    _extra_init);
+GST_BOILERPLATE (MpegTSParse, mpegts_parse, GstElement, GST_TYPE_ELEMENT);
 
 static const guint32 crc_tab[256] = {
   0x00000000, 0x04c11db7, 0x09823b6e, 0x0d4326d9, 0x130476dc, 0x17c56b6b,
@@ -206,33 +196,19 @@ mpegts_parse_calc_crc32 (guint8 * data, guint datalen)
   return crc;
 }
 
-static void
-_extra_init (GType type)
-{
-  QUARK_PROGRAMS = g_quark_from_string ("programs");
-  QUARK_PROGRAM_NUMBER = g_quark_from_string ("program-number");
-  QUARK_PID = g_quark_from_string ("pid");
-  QUARK_PCR_PID = g_quark_from_string ("pcr-pid");
-  QUARK_STREAMS = g_quark_from_string ("streams");
-  QUARK_STREAM_TYPE = g_quark_from_string ("stream-type");
-}
-
 static void
 mpegts_parse_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &sink_template);
-  gst_element_class_add_static_pad_template (element_class, &src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &program_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&program_template));
 
-  gst_element_class_set_details_simple (element_class,
-      "MPEG transport stream parser", "Codec/Parser",
-      "Parses MPEG2 transport streams",
-      "Alessandro Decina <alessandro@nnva.org>, "
-      "Zaheer Abbas Merali <zaheerabbas at merali dot org>");
+  gst_element_class_set_details (element_class, &mpegts_parse_details);
 }
 
 static void
@@ -256,8 +232,7 @@ mpegts_parse_class_init (MpegTSParseClass * klass)
   g_object_class_install_property (gobject_class, PROP_PROGRAM_NUMBERS,
       g_param_spec_string ("program-numbers",
           "Program Numbers",
-          "Colon separated list of programs", "",
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          "Colon separated list of programs", "", G_PARAM_READWRITE));
 }
 
 static gboolean
@@ -275,6 +250,9 @@ mpegts_parse_reset (MpegTSParse * parse)
   /* PAT */
   g_hash_table_insert (parse->psi_pids,
       GINT_TO_POINTER (0), GINT_TO_POINTER (1));
+  if (parse->pat != NULL)
+    gst_structure_free (parse->pat);
+  parse->pat = NULL;
   /* pmt pids will be added and removed dynamically */
 
 }
@@ -298,7 +276,6 @@ mpegts_parse_init (MpegTSParse * parse, MpegTSParseClass * klass)
   parse->psi_pids = g_hash_table_new (g_direct_hash, g_direct_equal);
   parse->pes_pids = g_hash_table_new (g_direct_hash, g_direct_equal);
   mpegts_parse_reset (parse);
-
 }
 
 static void
@@ -576,35 +553,6 @@ mpegts_parse_program_remove_stream (MpegTSParse * parse,
   g_hash_table_remove (program->streams, GINT_TO_POINTER ((gint) pid));
 }
 
-static void
-mpegts_parse_deactivate_pmt (MpegTSParse * parse, MpegTSParseProgram * program)
-{
-  gint i;
-  guint pid;
-  guint stream_type;
-  GstStructure *stream;
-  const GValue *streams;
-  const GValue *value;
-
-  if (program->pmt_info) {
-    streams = gst_structure_id_get_value (program->pmt_info, QUARK_STREAMS);
-
-    for (i = 0; i < gst_value_list_get_size (streams); ++i) {
-      value = gst_value_list_get_value (streams, i);
-      stream = g_value_get_boxed (value);
-      gst_structure_id_get (stream, QUARK_PID, G_TYPE_UINT, &pid,
-          QUARK_STREAM_TYPE, G_TYPE_UINT, &stream_type, NULL);
-      mpegts_parse_program_remove_stream (parse, program, (guint16) pid);
-      g_hash_table_remove (parse->pes_pids, GINT_TO_POINTER ((gint) pid));
-    }
-
-    /* remove pcr stream */
-    mpegts_parse_program_remove_stream (parse, program, program->pcr_pid);
-    g_hash_table_remove (parse->pes_pids,
-        GINT_TO_POINTER ((gint) program->pcr_pid));
-  }
-}
-
 static MpegTSParsePad *
 mpegts_parse_create_tspad (MpegTSParse * parse, const gchar * pad_name)
 {
@@ -630,10 +578,6 @@ mpegts_parse_create_tspad (MpegTSParse * parse, const gchar * pad_name)
 static void
 mpegts_parse_destroy_tspad (MpegTSParse * parse, MpegTSParsePad * tspad)
 {
-  if (tspad->tags) {
-    gst_tag_list_free (tspad->tags);
-  }
-
   /* free the wrapper */
   g_free (tspad);
 }
@@ -736,12 +680,6 @@ mpegts_parse_tspad_push (MpegTSParse * parse, MpegTSParsePad * tspad,
   if (tspad->program_number != -1) {
     if (tspad->program) {
       pad_pids = tspad->program->streams;
-
-      if (tspad->tags) {
-        gst_element_found_tags_for_pad (GST_ELEMENT_CAST (parse),
-            tspad->pad, tspad->tags);
-        tspad->tags = NULL;
-      }
     } else {
       /* there's a program filter on the pad but the PMT for the program has not
        * been parsed yet, ignore the pad until we get a PMT */
@@ -778,19 +716,21 @@ static GstFlowReturn
 mpegts_parse_push (MpegTSParse * parse, MpegTSPacketizerPacket * packet,
     MpegTSPacketizerSection * section)
 {
-  guint32 pads_cookie;
+  GstIterator *iterator;
   gboolean done = FALSE;
-  GstPad *pad = NULL;
+  gpointer pad = NULL;
   MpegTSParsePad *tspad;
   guint16 pid;
   GstBuffer *buffer;
   GstFlowReturn ret;
-  GList *srcpads;
+  GstCaps *caps;
 
   pid = packet->pid;
-  buffer = gst_buffer_make_metadata_writable (packet->buffer);
+  buffer = packet->buffer;
   /* we have the same caps on all the src pads */
-  gst_buffer_set_caps (buffer, parse->packetizer->caps);
+  caps = gst_static_pad_template_get_caps (&src_template);
+  gst_buffer_set_caps (buffer, caps);
+  gst_caps_unref (caps);
 
   GST_OBJECT_LOCK (parse);
   /* clear tspad->pushed on pads */
@@ -800,68 +740,54 @@ mpegts_parse_push (MpegTSParse * parse, MpegTSPacketizerPacket * packet,
     ret = GST_FLOW_NOT_LINKED;
   else
     ret = GST_FLOW_OK;
-
-  /* Get cookie and source pads list */
-  pads_cookie = GST_ELEMENT_CAST (parse)->pads_cookie;
-  srcpads = GST_ELEMENT_CAST (parse)->srcpads;
-  if (G_LIKELY (srcpads)) {
-    pad = GST_PAD_CAST (srcpads->data);
-    g_object_ref (pad);
-  }
   GST_OBJECT_UNLOCK (parse);
 
-  while (pad && !done) {
-    tspad = gst_pad_get_element_private (pad);
+  iterator = gst_element_iterate_src_pads (GST_ELEMENT_CAST (parse));
+  while (!done) {
+    switch (gst_iterator_next (iterator, &pad)) {
+      case GST_ITERATOR_OK:
+        tspad = gst_pad_get_element_private (GST_PAD (pad));
 
-    if (G_LIKELY (!tspad->pushed)) {
-      /* ref the buffer as gst_pad_push takes a ref but we want to reuse the
-       * same buffer for next pushes */
-      gst_buffer_ref (buffer);
-      if (section) {
-        tspad->flow_return =
-            mpegts_parse_tspad_push_section (parse, tspad, section, buffer);
-      } else {
-        tspad->flow_return =
-            mpegts_parse_tspad_push (parse, tspad, pid, buffer);
-      }
-      tspad->pushed = TRUE;
+        /* make sure to push only once if the iterator resyncs */
+        if (!tspad->pushed) {
+          /* ref the buffer as gst_pad_push takes a ref but we want to reuse the
+           * same buffer for next pushes */
+          gst_buffer_ref (buffer);
+          if (section) {
+            tspad->flow_return =
+                mpegts_parse_tspad_push_section (parse, tspad, section, buffer);
+          } else {
+            tspad->flow_return =
+                mpegts_parse_tspad_push (parse, tspad, pid, buffer);
+          }
+          tspad->pushed = TRUE;
 
-      if (G_UNLIKELY (tspad->flow_return != GST_FLOW_OK
-              && tspad->flow_return != GST_FLOW_NOT_LINKED)) {
-        /* return the error upstream */
-        ret = tspad->flow_return;
+          if (GST_FLOW_IS_FATAL (tspad->flow_return)) {
+            /* return the error upstream */
+            ret = tspad->flow_return;
+            done = TRUE;
+          }
+        }
+
+        if (ret == GST_FLOW_NOT_LINKED)
+          ret = tspad->flow_return;
+
+        /* the iterator refs the pad */
+        g_object_unref (GST_PAD (pad));
+        break;
+      case GST_ITERATOR_RESYNC:
+        gst_iterator_resync (iterator);
+        break;
+      case GST_ITERATOR_DONE:
         done = TRUE;
-      }
-
-    }
-
-    if (ret == GST_FLOW_NOT_LINKED)
-      ret = tspad->flow_return;
-
-    g_object_unref (pad);
-
-    if (G_UNLIKELY (!done)) {
-      GST_OBJECT_LOCK (parse);
-      if (G_UNLIKELY (pads_cookie != GST_ELEMENT_CAST (parse)->pads_cookie)) {
-        /* resync */
-        GST_DEBUG ("resync");
-        pads_cookie = GST_ELEMENT_CAST (parse)->pads_cookie;
-        srcpads = GST_ELEMENT_CAST (parse)->srcpads;
-      } else {
-        GST_DEBUG ("getting next pad");
-        /* Get next pad */
-        srcpads = g_list_next (srcpads);
-      }
-
-      if (srcpads) {
-        pad = GST_PAD_CAST (srcpads->data);
-        g_object_ref (pad);
-      } else
-        done = TRUE;
-      GST_OBJECT_UNLOCK (parse);
+        break;
+      default:
+        g_warning ("this should not be reached");
     }
   }
 
+  gst_iterator_free (iterator);
+
   gst_buffer_unref (buffer);
   packet->buffer = NULL;
 
@@ -873,15 +799,12 @@ mpegts_parse_is_psi (MpegTSParse * parse, MpegTSPacketizerPacket * packet)
 {
   gboolean retval = FALSE;
   guint8 table_id;
-  guint8 *data;
-  guint8 pointer;
   int i;
-  static const guint8 si_tables[] =
-      { 0x00, 0x01, 0x02, 0x03, 0x40, 0x41, 0x42, 0x46, 0x4A, 0x4E, 0x4F, 0x50,
-    0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x5B, 0x5C,
-    0x5D, 0x5E, 0x5F, 0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68,
-    0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70, 0x71, 0x72, 0x73, 0x7E,
-    0x7F, TABLE_ID_UNSET
+  guint8 si_tables[] = { 0x00, 0x01, 0x02, 0x03, 0x40, 0x41, 0x42, 0x46, 0x4A,
+    0x4E, 0x4F, 0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59,
+    0x5A, 0x5B, 0x5C, 0x5D, 0x5E, 0x5F, 0x60, 0x61, 0x62, 0x63, 0x64, 0x65,
+    0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70, 0x71,
+    0x72, 0x73, 0x7E, 0x7F, TABLE_ID_UNSET
   };
   if (g_hash_table_lookup (parse->psi_pids,
           GINT_TO_POINTER ((gint) packet->pid)) != NULL)
@@ -892,21 +815,10 @@ mpegts_parse_is_psi (MpegTSParse * parse, MpegTSPacketizerPacket * packet)
     return FALSE;
   if (!retval) {
     if (packet->payload_unit_start_indicator) {
-      data = packet->data;
-      pointer = *data++;
-      data += pointer;
-      /* 'pointer' value may be invalid on malformed packet
-       * so we need to avoid out of range
-       */
-      if (!(data < packet->data_end)) {
-        GST_WARNING_OBJECT (parse,
-            "Wrong offset when retrieving table id: 0x%x", pointer);
-        return FALSE;
-      }
-      table_id = *data;
+      table_id = *(packet->data);
       i = 0;
       while (si_tables[i] != TABLE_ID_UNSET) {
-        if (G_UNLIKELY (si_tables[i] == table_id)) {
+        if (si_tables[i] == table_id) {
           GST_DEBUG_OBJECT (parse, "Packet has table id 0x%x", table_id);
           retval = TRUE;
           break;
@@ -914,13 +826,16 @@ mpegts_parse_is_psi (MpegTSParse * parse, MpegTSPacketizerPacket * packet)
         i++;
       }
     } else {
-      MpegTSPacketizerStream *stream = parse->packetizer->streams[packet->pid];
+      MpegTSPacketizerStream *stream = (MpegTSPacketizerStream *)
+          g_hash_table_lookup (parse->packetizer->streams,
+          GINT_TO_POINTER ((gint) packet->pid));
+
       if (stream) {
         i = 0;
         GST_DEBUG_OBJECT (parse, "section table id: 0x%x",
             stream->section_table_id);
         while (si_tables[i] != TABLE_ID_UNSET) {
-          if (G_UNLIKELY (si_tables[i] == stream->section_table_id)) {
+          if (si_tables[i] == stream->section_table_id) {
             retval = TRUE;
             break;
           }
@@ -929,7 +844,6 @@ mpegts_parse_is_psi (MpegTSParse * parse, MpegTSPacketizerPacket * packet)
       }
     }
   }
-
   GST_DEBUG_OBJECT (parse, "Packet of pid 0x%x is psi: %d", packet->pid,
       retval);
   return retval;
@@ -946,25 +860,28 @@ mpegts_parse_apply_pat (MpegTSParse * parse, GstStructure * pat_info)
   MpegTSParseProgram *program;
   gint i;
   const GValue *programs;
+  gchar *dbg;
 
   old_pat = parse->pat;
   parse->pat = gst_structure_copy (pat_info);
 
-  GST_INFO_OBJECT (parse, "PAT %" GST_PTR_FORMAT, pat_info);
+  dbg = gst_structure_to_string (pat_info);
+  GST_INFO_OBJECT (parse, "PAT %s", dbg);
+  g_free (dbg);
 
   gst_element_post_message (GST_ELEMENT_CAST (parse),
       gst_message_new_element (GST_OBJECT (parse),
           gst_structure_copy (pat_info)));
 
   GST_OBJECT_LOCK (parse);
-  programs = gst_structure_id_get_value (pat_info, QUARK_PROGRAMS);
+  programs = gst_structure_get_value (pat_info, "programs");
   /* activate the new table */
   for (i = 0; i < gst_value_list_get_size (programs); ++i) {
     value = gst_value_list_get_value (programs, i);
 
     program_info = g_value_get_boxed (value);
-    gst_structure_id_get (program_info, QUARK_PROGRAM_NUMBER, G_TYPE_UINT,
-        &program_number, QUARK_PID, G_TYPE_UINT, &pid, NULL);
+    gst_structure_get_uint (program_info, "program-number", &program_number);
+    gst_structure_get_uint (program_info, "pid", &pid);
 
     program = mpegts_parse_get_program (parse, program_number);
     if (program) {
@@ -993,14 +910,13 @@ mpegts_parse_apply_pat (MpegTSParse * parse, GstStructure * pat_info)
   if (old_pat) {
     /* deactivate the old table */
 
-    programs = gst_structure_id_get_value (old_pat, QUARK_PROGRAMS);
+    programs = gst_structure_get_value (old_pat, "programs");
     for (i = 0; i < gst_value_list_get_size (programs); ++i) {
       value = gst_value_list_get_value (programs, i);
 
       program_info = g_value_get_boxed (value);
-      gst_structure_id_get (program_info,
-          QUARK_PROGRAM_NUMBER, G_TYPE_UINT, &program_number,
-          QUARK_PID, G_TYPE_UINT, &pid, NULL);
+      gst_structure_get_uint (program_info, "program-number", &program_number);
+      gst_structure_get_uint (program_info, "pid", &pid);
 
       program = mpegts_parse_get_program (parse, program_number);
       if (program == NULL) {
@@ -1013,14 +929,17 @@ mpegts_parse_apply_pat (MpegTSParse * parse, GstStructure * pat_info)
         /* the program has been referenced by the new pat, keep it */
         continue;
 
-      GST_INFO_OBJECT (parse, "PAT removing program %" GST_PTR_FORMAT,
-          program_info);
+      {
+        gchar *dbg = gst_structure_to_string (program_info);
+
+        GST_INFO_OBJECT (parse, "PAT removing program %s", dbg);
+        g_free (dbg);
+      }
 
       if (program->active)
         parse->pads_to_remove = g_list_append (parse->pads_to_remove,
             mpegts_parse_deactivate_program (parse, program));
 
-      mpegts_parse_deactivate_pmt (parse, program);
       mpegts_parse_remove_program (parse, program_number);
       g_hash_table_remove (parse->psi_pids, GINT_TO_POINTER ((gint) pid));
       mpegts_packetizer_remove_stream (parse->packetizer, pid);
@@ -1045,22 +964,38 @@ mpegts_parse_apply_pmt (MpegTSParse * parse,
   guint stream_type;
   GstStructure *stream;
   gint i;
+  const GValue *old_streams;
   const GValue *new_streams;
   const GValue *value;
 
-  gst_structure_id_get (pmt_info,
-      QUARK_PROGRAM_NUMBER, G_TYPE_UINT, &program_number,
-      QUARK_PCR_PID, G_TYPE_UINT, &pcr_pid, NULL);
-  new_streams = gst_structure_id_get_value (pmt_info, QUARK_STREAMS);
+  gst_structure_get_uint (pmt_info, "program-number", &program_number);
+  gst_structure_get_uint (pmt_info, "pcr-pid", &pcr_pid);
+  new_streams = gst_structure_get_value (pmt_info, "streams");
 
   GST_OBJECT_LOCK (parse);
   program = mpegts_parse_get_program (parse, program_number);
   if (program) {
-    /* deactivate old pmt */
-    mpegts_parse_deactivate_pmt (parse, program);
-    if (program->pmt_info)
+    if (program->pmt_info) {
+      /* deactivate old pmt */
+      old_streams = gst_structure_get_value (program->pmt_info, "streams");
+
+      for (i = 0; i < gst_value_list_get_size (old_streams); ++i) {
+        value = gst_value_list_get_value (old_streams, i);
+        stream = g_value_get_boxed (value);
+        gst_structure_get_uint (stream, "pid", &pid);
+        gst_structure_get_uint (stream, "stream-type", &stream_type);
+        mpegts_parse_program_remove_stream (parse, program, (guint16) pid);
+        g_hash_table_remove (parse->pes_pids, GINT_TO_POINTER ((gint) pid));
+      }
+
+      /* remove pcr stream */
+      mpegts_parse_program_remove_stream (parse, program, program->pcr_pid);
+      g_hash_table_remove (parse->pes_pids,
+          GINT_TO_POINTER ((gint) program->pcr_pid));
+
       gst_structure_free (program->pmt_info);
-    program->pmt_info = NULL;
+      program->pmt_info = NULL;
+    }
   } else {
     /* no PAT?? */
     g_hash_table_insert (parse->psi_pids,
@@ -1080,8 +1015,8 @@ mpegts_parse_apply_pmt (MpegTSParse * parse,
     value = gst_value_list_get_value (new_streams, i);
     stream = g_value_get_boxed (value);
 
-    gst_structure_id_get (stream, QUARK_PID, G_TYPE_UINT, &pid,
-        QUARK_STREAM_TYPE, G_TYPE_UINT, &stream_type, NULL);
+    gst_structure_get_uint (stream, "pid", &pid);
+    gst_structure_get_uint (stream, "stream-type", &stream_type);
     mpegts_parse_program_add_stream (parse, program,
         (guint16) pid, (guint8) stream_type);
     g_hash_table_insert (parse->pes_pids, GINT_TO_POINTER ((gint) pid),
@@ -1090,7 +1025,12 @@ mpegts_parse_apply_pmt (MpegTSParse * parse,
   }
   GST_OBJECT_UNLOCK (parse);
 
-  GST_DEBUG_OBJECT (parse, "new pmt %" GST_PTR_FORMAT, pmt_info);
+  {
+    gchar *dbg = gst_structure_to_string (pmt_info);
+
+    GST_DEBUG_OBJECT (parse, "new pmt %s", dbg);
+    g_free (dbg);
+  }
 
   gst_element_post_message (GST_ELEMENT_CAST (parse),
       gst_message_new_element (GST_OBJECT (parse),
@@ -1110,8 +1050,6 @@ static void
 mpegts_parse_apply_sdt (MpegTSParse * parse,
     guint16 pmt_pid, GstStructure * sdt_info)
 {
-  mpegts_parse_get_tags_from_sdt (parse, sdt_info);
-
   gst_element_post_message (GST_ELEMENT_CAST (parse),
       gst_message_new_element (GST_OBJECT (parse),
           gst_structure_copy (sdt_info)));
@@ -1121,46 +1059,28 @@ static void
 mpegts_parse_apply_eit (MpegTSParse * parse,
     guint16 pmt_pid, GstStructure * eit_info)
 {
-  mpegts_parse_get_tags_from_eit (parse, eit_info);
-
   gst_element_post_message (GST_ELEMENT_CAST (parse),
       gst_message_new_element (GST_OBJECT (parse),
           gst_structure_copy (eit_info)));
 }
 
-static void
-mpegts_parse_apply_tdt (MpegTSParse * parse,
-    guint16 tdt_pid, GstStructure * tdt_info)
-{
-  gst_element_post_message (GST_ELEMENT_CAST (parse),
-      gst_message_new_element (GST_OBJECT (parse),
-          gst_structure_copy (tdt_info)));
-
-  gst_element_send_event (GST_ELEMENT_CAST (parse),
-      gst_event_new_custom (GST_EVENT_CUSTOM_DOWNSTREAM,
-          gst_structure_copy (tdt_info)));
-}
-
 static gboolean
 mpegts_parse_handle_psi (MpegTSParse * parse, MpegTSPacketizerSection * section)
 {
   gboolean res = TRUE;
   GstStructure *structure = NULL;
 
-  /* table ids 0x70 - 0x73 do not have a crc */
-  if (G_LIKELY (section->table_id < 0x70 || section->table_id > 0x73)) {
-    if (G_UNLIKELY (mpegts_parse_calc_crc32 (GST_BUFFER_DATA (section->buffer),
-                GST_BUFFER_SIZE (section->buffer)) != 0)) {
-      GST_WARNING_OBJECT (parse, "bad crc in psi pid 0x%x", section->pid);
-      return FALSE;
-    }
+  if (mpegts_parse_calc_crc32 (GST_BUFFER_DATA (section->buffer),
+          GST_BUFFER_SIZE (section->buffer)) != 0) {
+    GST_WARNING_OBJECT (parse, "bad crc in psi pid 0x%x", section->pid);
+    return FALSE;
   }
 
   switch (section->table_id) {
     case 0x00:
       /* PAT */
       structure = mpegts_packetizer_parse_pat (parse->packetizer, section);
-      if (G_LIKELY (structure))
+      if (structure)
         mpegts_parse_apply_pat (parse, structure);
       else
         res = FALSE;
@@ -1168,7 +1088,7 @@ mpegts_parse_handle_psi (MpegTSParse * parse, MpegTSPacketizerSection * section)
       break;
     case 0x02:
       structure = mpegts_packetizer_parse_pmt (parse->packetizer, section);
-      if (G_LIKELY (structure))
+      if (structure)
         mpegts_parse_apply_pmt (parse, section->pid, structure);
       else
         res = FALSE;
@@ -1179,7 +1099,7 @@ mpegts_parse_handle_psi (MpegTSParse * parse, MpegTSPacketizerSection * section)
     case 0x41:
       /* NIT, other network */
       structure = mpegts_packetizer_parse_nit (parse->packetizer, section);
-      if (G_LIKELY (structure))
+      if (structure)
         mpegts_parse_apply_nit (parse, section->pid, structure);
       else
         res = FALSE;
@@ -1188,7 +1108,7 @@ mpegts_parse_handle_psi (MpegTSParse * parse, MpegTSPacketizerSection * section)
     case 0x42:
     case 0x46:
       structure = mpegts_packetizer_parse_sdt (parse->packetizer, section);
-      if (G_LIKELY (structure))
+      if (structure)
         mpegts_parse_apply_sdt (parse, section->pid, structure);
       else
         res = FALSE;
@@ -1230,19 +1150,11 @@ mpegts_parse_handle_psi (MpegTSParse * parse, MpegTSPacketizerSection * section)
     case 0x6F:
       /* EIT, schedule */
       structure = mpegts_packetizer_parse_eit (parse->packetizer, section);
-      if (G_LIKELY (structure))
+      if (structure)
         mpegts_parse_apply_eit (parse, section->pid, structure);
       else
         res = FALSE;
       break;
-    case 0x70:
-      /* TDT (Time and Date table) */
-      structure = mpegts_packetizer_parse_tdt (parse->packetizer, section);
-      if (G_LIKELY (structure))
-        mpegts_parse_apply_tdt (parse, section->pid, structure);
-      else
-        res = FALSE;
-      break;
     default:
       break;
   }
@@ -1253,81 +1165,6 @@ mpegts_parse_handle_psi (MpegTSParse * parse, MpegTSPacketizerSection * section)
   return res;
 }
 
-static void
-mpegts_parse_get_tags_from_sdt (MpegTSParse * parse, GstStructure * sdt_info)
-{
-  const GValue *services;
-  guint i;
-
-  services = gst_structure_get_value (sdt_info, "services");
-
-  for (i = 0; i < gst_value_list_get_size (services); i++) {
-    const GstStructure *service;
-    const gchar *sid_str;
-    gchar *tmp;
-    gint program_number;
-    MpegTSParseProgram *program;
-
-    service = gst_value_get_structure (gst_value_list_get_value (services, i));
-
-    /* get program_number from structure name
-     * which looks like service-%d */
-    sid_str = gst_structure_get_name (service);
-    tmp = g_strstr_len (sid_str, -1, "-");
-    if (!tmp)
-      continue;
-    program_number = atoi (++tmp);
-
-    program = mpegts_parse_get_program (parse, program_number);
-    if (program && program->tspad && !program->tspad->tags) {
-      program->tspad->tags = gst_tag_list_new_full (GST_TAG_ARTIST,
-          gst_structure_get_string (service, "name"), NULL);
-    }
-  }
-}
-
-static void
-mpegts_parse_get_tags_from_eit (MpegTSParse * parse, GstStructure * eit_info)
-{
-  const GValue *events;
-  guint i;
-  guint program_number;
-  MpegTSParseProgram *program;
-  gboolean present_following;
-
-  gst_structure_get_uint (eit_info, "service-id", &program_number);
-  program = mpegts_parse_get_program (parse, program_number);
-
-  gst_structure_get_boolean (eit_info, "present-following", &present_following);
-
-  if (program && program->tspad && present_following) {
-    events = gst_structure_get_value (eit_info, "events");
-
-    for (i = 0; i < gst_value_list_get_size (events); i++) {
-      const GstStructure *event;
-      const gchar *title;
-      guint status;
-      guint event_id;
-      guint duration;
-
-      event = gst_value_get_structure (gst_value_list_get_value (events, i));
-
-      title = gst_structure_get_string (event, "name");
-      gst_structure_get_uint (event, "event-id", &event_id);
-      gst_structure_get_uint (event, "running-status", &status);
-
-      if (title && event_id != program->tspad->event_id
-          && status == RUNNING_STATUS_RUNNING) {
-        gst_structure_get_uint (event, "duration", &duration);
-
-        program->tspad->event_id = event_id;
-        program->tspad->tags = gst_tag_list_new_full (GST_TAG_TITLE,
-            title, GST_TAG_DURATION, duration * GST_SECOND, NULL);
-      }
-    }
-  }
-}
-
 static gboolean
 mpegts_parse_sink_event (GstPad * pad, GstEvent * event)
 {
@@ -1354,7 +1191,6 @@ mpegts_parse_chain (GstPad * pad, GstBuffer * buf)
   GstFlowReturn res = GST_FLOW_OK;
   MpegTSParse *parse;
   gboolean parsed;
-  MpegTSPacketizerPacketReturn pret;
   MpegTSPacketizer *packetizer;
   MpegTSPacketizerPacket packet;
 
@@ -1362,10 +1198,11 @@ mpegts_parse_chain (GstPad * pad, GstBuffer * buf)
   packetizer = parse->packetizer;
 
   mpegts_packetizer_push (parse->packetizer, buf);
-  while (((pret =
-              mpegts_packetizer_next_packet (parse->packetizer,
-                  &packet)) != PACKET_NEED_MORE) && res == GST_FLOW_OK) {
-    if (G_UNLIKELY (pret == PACKET_BAD))
+  while (mpegts_packetizer_has_packets (parse->packetizer) &&
+      !GST_FLOW_IS_FATAL (res)) {
+    /* get the next packet */
+    parsed = mpegts_packetizer_next_packet (packetizer, &packet);
+    if (!parsed)
       /* bad header, skip the packet */
       goto next;
 
@@ -1374,16 +1211,16 @@ mpegts_parse_chain (GstPad * pad, GstBuffer * buf)
       MpegTSPacketizerSection section;
 
       parsed = mpegts_packetizer_push_section (packetizer, &packet, &section);
-      if (G_UNLIKELY (!parsed))
+      if (!parsed)
         /* bad section data */
         goto next;
 
-      if (G_LIKELY (section.complete)) {
+      if (section.complete) {
         /* section complete */
         parsed = mpegts_parse_handle_psi (parse, &section);
         gst_buffer_unref (section.buffer);
 
-        if (G_UNLIKELY (!parsed))
+        if (!parsed)
           /* bad PSI table */
           goto next;
       }
@@ -1464,6 +1301,7 @@ gst_mpegtsparse_plugin_init (GstPlugin * plugin)
   GST_DEBUG_CATEGORY_INIT (mpegts_parse_debug, "mpegtsparse", 0,
       "MPEG transport stream parser");
 
+  mpegts_packetizer_init_debug ();
   gst_mpegtsdesc_init_debug ();
 
   return gst_element_register (plugin, "mpegtsparse",
