@@ -41,9 +41,17 @@
  *     sequence changes, but if we then seek to a different spot it might
  *     be wrong. Fortunately almost every stream only has 1 sequence.
  */
-GST_DEBUG_CATEGORY (mpv_parse_debug);
+GST_DEBUG_CATEGORY_STATIC (mpv_parse_debug);
 #define GST_CAT_DEFAULT mpv_parse_debug
 
+/* elementfactory information */
+static GstElementDetails mpegvideoparse_details =
+GST_ELEMENT_DETAILS ("MPEG video elementary stream parser",
+    "Codec/Parser/Video",
+    "Parses and frames MPEG-1 and MPEG-2 elementary video streams",
+    "Wim Taymans <wim.taymans@chello.be>\n"
+    "Jan Schmidt <thaytan@mad.scientist.com>");
+
 static GstStaticPadTemplate src_template =
 GST_STATIC_PAD_TEMPLATE ("src", GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -127,16 +135,12 @@ gst_mpegvideoparse_base_init (MpegVideoParseClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class, &src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
 
-  gst_element_class_set_details_simple (element_class,
-      "MPEG video elementary stream parser",
-      "Codec/Parser/Video",
-      "Parses and frames MPEG-1 and MPEG-2 elementary video streams",
-      "Wim Taymans <wim.taymans@chello.be>, "
-      "Jan Schmidt <thaytan@mad.scientist.com>");
+  gst_element_class_set_details (element_class, &mpegvideoparse_details);
 }
 
 static void
@@ -239,27 +243,9 @@ mpegvideoparse_handle_sequence (MpegVideoParse * mpegvideoparse,
   if (G_UNLIKELY (!mpeg_util_parse_sequence_hdr (&new_hdr, cur, end)))
     return FALSE;
 
-  if (new_hdr.width < 16 || new_hdr.width > 4096 ||
-      new_hdr.height < 16 || new_hdr.height > 4096) {
-    GST_WARNING_OBJECT (mpegvideoparse, "Width/height out of valid range "
-        "[16, 4096]");
-    return FALSE;
-  }
-
   if (memcmp (&mpegvideoparse->seq_hdr, &new_hdr, sizeof (MPEGSeqHdr)) != 0) {
     GstCaps *caps;
     GstBuffer *seq_buf;
-    /*
-     * Profile indication - 1 => High, 2 => Spatially Scalable,
-     *                      3 => SNR Scalable, 4 => Main, 5 => Simple
-     * 4:2:2 and Multi-view have profile = 0, with the escape bit set to 1
-     */
-    const gchar *profiles[] = { "high", "spatial", "snr", "main", "simple" };
-    /*
-     * Level indication - 4 => High, 6 => High-1440, 8 => Main, 10 => Low,
-     *                    except in the case of profile = 0
-     */
-    const gchar *levels[] = { "high", "high-1440", "main", "low" };
 
     /* Store the entire sequence header + sequence header extension
        for output as codec_data */
@@ -275,58 +261,8 @@ mpegvideoparse_handle_sequence (MpegVideoParse * mpegvideoparse,
         "height", G_TYPE_INT, new_hdr.height,
         "framerate", GST_TYPE_FRACTION, new_hdr.fps_n, new_hdr.fps_d,
         "pixel-aspect-ratio", GST_TYPE_FRACTION, new_hdr.par_w, new_hdr.par_h,
-        "interlaced", G_TYPE_BOOLEAN, !new_hdr.progressive,
         "codec_data", GST_TYPE_BUFFER, seq_buf, NULL);
 
-    if (new_hdr.mpeg_version == 2) {
-      const gchar *profile = NULL, *level = NULL;
-
-      if (new_hdr.profile > 0 && new_hdr.profile < 6)
-        profile = profiles[new_hdr.profile - 1];
-
-      if ((new_hdr.level > 3) && (new_hdr.level < 11) &&
-          (new_hdr.level % 2 == 0))
-        level = levels[(new_hdr.level >> 1) - 1];
-
-      if (new_hdr.profile == 8) {
-        /* Non-hierarchical profile */
-        switch (new_hdr.level) {
-          case 2:
-            level = levels[0];
-            break;
-          case 5:
-            level = levels[2];
-            profile = "4:2:2";
-            break;
-          case 10:
-            level = levels[0];
-            break;
-          case 11:
-            level = levels[1];
-            break;
-          case 13:
-            level = levels[2];
-            break;
-          case 14:
-            level = levels[3];
-            profile = "multiview";
-            break;
-          default:
-            break;
-        }
-      }
-
-      if (profile)
-        gst_caps_set_simple (caps, "profile", G_TYPE_STRING, profile, NULL);
-      else
-        GST_DEBUG ("Invalid profile - %u", new_hdr.profile);
-
-      if (level)
-        gst_caps_set_simple (caps, "level", G_TYPE_STRING, level, NULL);
-      else
-        GST_DEBUG ("Invalid level - %u", new_hdr.level);
-    }
-
     GST_DEBUG ("New mpegvideoparse caps: %" GST_PTR_FORMAT, caps);
     if (!gst_pad_set_caps (mpegvideoparse->srcpad, caps)) {
       gst_caps_unref (caps);
@@ -334,14 +270,6 @@ mpegvideoparse_handle_sequence (MpegVideoParse * mpegvideoparse,
     }
     gst_caps_unref (caps);
 
-    if (new_hdr.bitrate > 0) {
-      GstTagList *taglist;
-
-      taglist = gst_tag_list_new_full (GST_TAG_BITRATE, new_hdr.bitrate, NULL);
-      gst_element_found_tags_for_pad (GST_ELEMENT_CAST (mpegvideoparse),
-          mpegvideoparse->srcpad, taglist);
-    }
-
     /* And update the new_hdr into our stored version */
     mpegvideoparse->seq_hdr = new_hdr;
   }
@@ -349,64 +277,6 @@ mpegvideoparse_handle_sequence (MpegVideoParse * mpegvideoparse,
   return TRUE;
 }
 
-#ifndef GST_DISABLE_GST_DEBUG
-static const gchar *
-picture_start_code_name (guint8 psc)
-{
-  guint i;
-  const struct
-  {
-    guint8 psc;
-    const gchar *name;
-  } psc_names[] = {
-    {
-    0x00, "Picture Start"}, {
-    0xb0, "Reserved"}, {
-    0xb1, "Reserved"}, {
-    0xb2, "User Data Start"}, {
-    0xb3, "Sequence Header Start"}, {
-    0xb4, "Sequence Error"}, {
-    0xb5, "Extnsion Start"}, {
-    0xb6, "Reserved"}, {
-    0xb7, "Sequence End"}, {
-    0xb8, "Group Start"}, {
-    0xb9, "Program End"}
-  };
-  if (psc < 0xB0 && psc > 0)
-    return "Slice Start";
-
-  for (i = 0; i < G_N_ELEMENTS (psc_names); i++)
-    if (psc_names[i].psc == psc)
-      return psc_names[i].name;
-
-  return "UNKNOWN";
-};
-
-static const gchar *
-picture_type_name (guint8 pct)
-{
-  guint i;
-  const struct
-  {
-    guint8 pct;
-    const gchar *name;
-  } pct_names[] = {
-    {
-    0, "Forbidden"}, {
-    1, "I Frame"}, {
-    2, "P Frame"}, {
-    3, "B Frame"}, {
-    4, "DC Intra Coded (Shall Not Be Used!)"}
-  };
-
-  for (i = 0; i < G_N_ELEMENTS (pct_names); i++)
-    if (pct_names[i].pct == pct)
-      return pct_names[i].name;
-
-  return "Reserved/Unknown";
-}
-#endif /* GST_DISABLE_GST_DEBUG */
-
 static gboolean
 mpegvideoparse_handle_picture (MpegVideoParse * mpegvideoparse, GstBuffer * buf)
 {
@@ -418,9 +288,6 @@ mpegvideoparse_handle_picture (MpegVideoParse * mpegvideoparse, GstBuffer * buf)
 
   cur = mpeg_util_find_start_code (&sync_word, cur, end);
   while (cur != NULL) {
-    if (cur[0] == 0 || cur[0] > 0xaf)
-      GST_LOG_OBJECT (mpegvideoparse, "Picture Start Code : %s",
-          picture_start_code_name (cur[0]));
     /* Cur points at the last byte of the start code */
     if (cur[0] == MPEG_PACKET_PICTURE) {
       guint8 *pic_data = cur - 3;
@@ -433,11 +300,9 @@ mpegvideoparse_handle_picture (MpegVideoParse * mpegvideoparse, GstBuffer * buf)
       if (hdr.pic_type != MPEG_PICTURE_TYPE_I)
         GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DELTA_UNIT);
 
-      GST_LOG_OBJECT (mpegvideoparse, "Picture type is %s",
-          picture_type_name (hdr.pic_type));
+      GST_LOG_OBJECT (mpegvideoparse, "Picture type is %u", hdr.pic_type);
       /* FIXME: Can use the picture type and number of fields to track a
        * timestamp */
-      break;
     }
     cur = mpeg_util_find_start_code (&sync_word, cur, end);
   }
@@ -493,25 +358,28 @@ mpegvideoparse_drain_avail (MpegVideoParse * mpegvideoparse)
   while ((cur != NULL) && (res == GST_FLOW_OK)) {
     /* Handle the block */
     GST_LOG_OBJECT (mpegvideoparse,
-        "Have block of size %u with pack_type %s and flags 0x%02x",
-        cur->length, picture_start_code_name (cur->first_pack_type),
-        cur->flags);
+        "Have block of size %u with pack_type 0x%02x and flags 0x%02x",
+        cur->length, cur->first_pack_type, cur->flags);
 
-    if ((cur->flags & MPEG_BLOCK_FLAG_SEQUENCE) && buf != NULL) {
-      /* Found a sequence header */
-      if (!mpegvideoparse_handle_sequence (mpegvideoparse, buf)) {
-        GST_DEBUG_OBJECT (mpegvideoparse,
-            "Invalid sequence header. Dropping buffer.");
-        gst_buffer_unref (buf);
-        buf = NULL;
+    /* Don't start pushing out buffers until we've seen a sequence header */
+    if (mpegvideoparse->seq_hdr.mpeg_version == 0) {
+      if (cur->flags & MPEG_BLOCK_FLAG_SEQUENCE) {
+        /* Found a sequence header */
+        if (!mpegvideoparse_handle_sequence (mpegvideoparse, buf)) {
+          GST_DEBUG_OBJECT (mpegvideoparse,
+              "Invalid sequence header. Dropping buffer.");
+          gst_buffer_unref (buf);
+          buf = NULL;
+        }
+      } else {
+        if (buf) {
+          GST_DEBUG_OBJECT (mpegvideoparse,
+              "No sequence header yet. Dropping buffer of %u bytes",
+              GST_BUFFER_SIZE (buf));
+          gst_buffer_unref (buf);
+          buf = NULL;
+        }
       }
-    } else if (mpegvideoparse->seq_hdr.mpeg_version == 0 && buf) {
-      /* Don't start pushing out buffers until we've seen a sequence header */
-      GST_DEBUG_OBJECT (mpegvideoparse,
-          "No sequence header yet. Dropping buffer of %u bytes",
-          GST_BUFFER_SIZE (buf));
-      gst_buffer_unref (buf);
-      buf = NULL;
     }
 
     if (buf != NULL) {
@@ -667,8 +535,7 @@ scan_keyframe (MpegVideoParse * mpegvideoparse)
   if (count)
     scanword = (scanword << (8 * (8 - count)));
 
-  GST_LOG_OBJECT (mpegvideoparse, "scanword 0x%016" G_GINT64_MODIFIER "x",
-      scanword);
+  GST_LOG_OBJECT (mpegvideoparse, "scanword 0x%016llx", scanword);
 
   data = GST_BUFFER_DATA (head);
   size = GST_BUFFER_SIZE (head);
@@ -676,8 +543,8 @@ scan_keyframe (MpegVideoParse * mpegvideoparse)
   while (size > 0) {
     scanword = (((guint64) data[size - 1]) << 56) | (scanword >> 8);
 
-    GST_LOG_OBJECT (mpegvideoparse,
-        "scanword at %d 0x%016" G_GINT64_MODIFIER "x", size - 1, scanword);
+    GST_LOG_OBJECT (mpegvideoparse, "scanword at %d 0x%016llx", size - 1,
+        scanword);
 
     /* check picture start and picture type */
     if ((scanword & G_GUINT64_CONSTANT (0xffffffff00380000)) ==
@@ -1026,11 +893,11 @@ gst_mpegvideoparse_change_state (GstElement * element,
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  GST_DEBUG_CATEGORY_INIT (mpv_parse_debug, "legacympegvideoparse", 0,
+  GST_DEBUG_CATEGORY_INIT (mpv_parse_debug, "mpegvideoparse", 0,
       "MPEG Video Parser");
 
-  return gst_element_register (plugin, "legacympegvideoparse",
-      GST_RANK_NONE, GST_TYPE_MPEGVIDEOPARSE);
+  return gst_element_register (plugin, "mpegvideoparse",
+      GST_RANK_SECONDARY - 1, GST_TYPE_MPEGVIDEOPARSE);
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
