@@ -2,10 +2,8 @@
  *
  * gstrtpmux.c:
  *
- * Copyright (C) <2007-2010> Nokia Corporation.
+ * Copyright (C) <2007> Nokia Corporation.
  *   Contact: Zeeshan Ali <zeeshan.ali@nokia.com>
- * Copyright (C) <2007-2010> Collabora Ltd
- *   Contact: Olivier Crete <olivier.crete@collabora.co.uk>
  * Copyright (C) 1999,2000 Erik Walthinsen <omega@cse.ogi.edu>
  *               2000,2005 Wim Taymans <wim@fluendo.com>
  *
@@ -27,27 +25,9 @@
 
 /**
  * SECTION:element-rtpmux
- * @see_also: rtpdtmfmux
  *
  * The rtp muxer takes multiple RTP streams having the same clock-rate and
  * muxes into a single stream with a single SSRC.
- *
- * <refsect2>
- * <title>Example pipelines</title>
- * |[
- * gst-launch rtpmux name=mux ! udpsink host=127.0.0.1 port=8888        \
- *              alsasrc ! alawenc ! rtppcmapay !                        \
- *              application/x-rtp, payload=8, rate=8000 ! mux.sink_0    \
- *              audiotestsrc is-live=1 !                                \
- *              mulawenc ! rtppcmupay !                                 \
- *              application/x-rtp, payload=0, rate=8000 ! mux.sink_1
- * ]|
- * In this example, an audio stream is captured from ALSA and another is
- * generated, both are encoded into different payload types and muxed together
- * so they can be sent on the same port.
- * </refsect2>
- *
- * Last reviewed on 2010-09-30 (0.10.21)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -63,6 +43,13 @@
 GST_DEBUG_CATEGORY_STATIC (gst_rtp_mux_debug);
 #define GST_CAT_DEFAULT gst_rtp_mux_debug
 
+/* elementfactory information */
+static const GstElementDetails gst_rtp_mux_details =
+GST_ELEMENT_DETAILS ("RTP muxer",
+    "Codec/Muxer",
+    "multiplex N rtp streams into one",
+    "Zeeshan Ali <first.last@nokia.com>");
+
 enum
 {
   ARG_0,
@@ -76,6 +63,14 @@ enum
 #define DEFAULT_SEQNUM_OFFSET    -1
 #define DEFAULT_SSRC             -1
 
+typedef struct
+{
+  gboolean have_clock_base;
+  guint clock_base;
+
+  GstCaps *out_caps;
+} GstRTPMuxPadPrivate;
+
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -88,15 +83,13 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink_%d",
     GST_STATIC_CAPS ("application/x-rtp")
     );
 
+static void gst_rtp_mux_finalize (GObject * object);
+
 static GstPad *gst_rtp_mux_request_new_pad (GstElement * element,
     GstPadTemplate * templ, const gchar * name);
-static void gst_rtp_mux_release_pad (GstElement * element, GstPad * pad);
 static GstFlowReturn gst_rtp_mux_chain (GstPad * pad, GstBuffer * buffer);
-static GstFlowReturn gst_rtp_mux_chain_list (GstPad * pad,
-    GstBufferList * bufferlist);
 static gboolean gst_rtp_mux_setcaps (GstPad * pad, GstCaps * caps);
 static GstCaps *gst_rtp_mux_getcaps (GstPad * pad);
-static gboolean gst_rtp_mux_sink_event (GstPad * pad, GstEvent * event);
 
 static GstStateChangeReturn gst_rtp_mux_change_state (GstElement *
     element, GstStateChange transition);
@@ -105,10 +98,7 @@ static void gst_rtp_mux_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_rtp_mux_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
-static void gst_rtp_mux_dispose (GObject * object);
 
-static gboolean gst_rtp_mux_src_event_real (GstRTPMux *rtp_mux,
-    GstEvent * event);
 
 GST_BOILERPLATE (GstRTPMux, gst_rtp_mux, GstElement, GST_TYPE_ELEMENT);
 
@@ -117,12 +107,12 @@ gst_rtp_mux_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class, &src_factory);
-  gst_element_class_add_static_pad_template (element_class, &sink_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_factory));
 
-  gst_element_class_set_details_simple (element_class, "RTP muxer",
-      "Codec/Muxer",
-      "multiplex N rtp streams into one", "Zeeshan Ali <first.last@nokia.com>");
+  gst_element_class_set_details (element_class, &gst_rtp_mux_details);
 }
 
 static void
@@ -134,82 +124,48 @@ gst_rtp_mux_class_init (GstRTPMuxClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
+  gobject_class->finalize = gst_rtp_mux_finalize;
   gobject_class->get_property = gst_rtp_mux_get_property;
   gobject_class->set_property = gst_rtp_mux_set_property;
-  gobject_class->dispose = gst_rtp_mux_dispose;
-
-  klass->src_event = gst_rtp_mux_src_event_real;
 
   g_object_class_install_property (G_OBJECT_CLASS (klass),
       PROP_TIMESTAMP_OFFSET, g_param_spec_int ("timestamp-offset",
           "Timestamp Offset",
           "Offset to add to all outgoing timestamps (-1 = random)", -1,
-          G_MAXINT, DEFAULT_TIMESTAMP_OFFSET,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_MAXINT, DEFAULT_TIMESTAMP_OFFSET, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SEQNUM_OFFSET,
       g_param_spec_int ("seqnum-offset", "Sequence number Offset",
           "Offset to add to all outgoing seqnum (-1 = random)", -1, G_MAXINT,
-          DEFAULT_SEQNUM_OFFSET, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          DEFAULT_SEQNUM_OFFSET, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SEQNUM,
       g_param_spec_uint ("seqnum", "Sequence number",
           "The RTP sequence number of the last processed packet",
-          0, G_MAXUINT, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+          0, G_MAXUINT, 0, G_PARAM_READABLE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SSRC,
       g_param_spec_uint ("ssrc", "SSRC",
           "The SSRC of the packets (-1 == random)",
-          0, G_MAXUINT, DEFAULT_SSRC,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          0, G_MAXUINT, DEFAULT_SSRC, G_PARAM_READWRITE));
 
   gstelement_class->request_new_pad =
       GST_DEBUG_FUNCPTR (gst_rtp_mux_request_new_pad);
-  gstelement_class->release_pad = GST_DEBUG_FUNCPTR (gst_rtp_mux_release_pad);
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_rtp_mux_change_state);
-}
 
-static void
-gst_rtp_mux_dispose (GObject * object)
-{
-  GList *item;
-
-restart:
-  for (item = GST_ELEMENT_PADS (object); item; item = g_list_next (item)) {
-    GstPad *pad = GST_PAD (item->data);
-    if (GST_PAD_IS_SINK (pad)) {
-      gst_element_release_request_pad (GST_ELEMENT (object), pad);
-      goto restart;
-    }
-  }
-
-  G_OBJECT_CLASS (parent_class)->dispose (object);
+  klass->chain_func = gst_rtp_mux_chain;
 }
 
 static gboolean
 gst_rtp_mux_src_event (GstPad * pad, GstEvent * event)
 {
-  GstRTPMux *rtp_mux;
-  GstRTPMuxClass *klass;
-  gboolean ret = FALSE;
-
-  rtp_mux = (GstRTPMux *) gst_pad_get_parent_element (pad);
-  g_return_val_if_fail (rtp_mux != NULL, FALSE);
-  klass = GST_RTP_MUX_GET_CLASS (rtp_mux);
-
-  ret = klass->src_event (rtp_mux, event);
-
-  gst_object_unref (rtp_mux);
-
-  return ret;
-}
-
-static gboolean
-gst_rtp_mux_src_event_real (GstRTPMux *rtp_mux, GstEvent * event)
-{
+  GstElement *rtp_mux;
   GstIterator *iter;
   GstPad *sinkpad;
   gboolean result = FALSE;
   gboolean done = FALSE;
 
-  iter = gst_element_iterate_sink_pads (GST_ELEMENT (rtp_mux));
+  rtp_mux = gst_pad_get_parent_element (pad);
+  g_return_val_if_fail (rtp_mux != NULL, FALSE);
+
+  iter = gst_element_iterate_sink_pads (rtp_mux);
 
   while (!done) {
     switch (gst_iterator_next (iter, (gpointer) & sinkpad)) {
@@ -230,6 +186,7 @@ gst_rtp_mux_src_event_real (GstRTPMux *rtp_mux, GstEvent * event)
     }
   }
   gst_iterator_free (iter);
+  gst_object_unref (rtp_mux);
   gst_event_unref (event);
 
   return result;
@@ -250,31 +207,76 @@ gst_rtp_mux_init (GstRTPMux * object, GstRTPMuxClass * g_class)
   object->ssrc = DEFAULT_SSRC;
   object->ts_offset = DEFAULT_TIMESTAMP_OFFSET;
   object->seqnum_offset = DEFAULT_SEQNUM_OFFSET;
+}
 
-  object->segment_pending = TRUE;
-  object->last_stop = GST_CLOCK_TIME_NONE;
+static void
+gst_rtp_mux_finalize (GObject * object)
+{
+  GstRTPMux *rtp_mux;
+
+  rtp_mux = GST_RTP_MUX (object);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+static GstPad *
+gst_rtp_mux_create_sinkpad (GstRTPMux * rtp_mux, GstPadTemplate * templ)
+{
+  GstPad *newpad = NULL;
+  GstPadTemplate *class_templ;
+
+  class_templ =
+      gst_element_class_get_pad_template (GST_ELEMENT_GET_CLASS (rtp_mux),
+      "sink_%d");
+
+  if (templ == class_templ) {
+    gchar *name;
+
+    /* create new pad with the name */
+    name = g_strdup_printf ("sink_%02d", rtp_mux->numpads);
+    newpad = gst_pad_new_from_template (templ, name);
+    g_free (name);
+
+    rtp_mux->numpads++;
+  } else {
+    GST_WARNING_OBJECT (rtp_mux, "this is not our template!\n");
+  }
+
+  return newpad;
+}
+
+static void
+free_pad_private (gpointer data, GObject * where_the_object_was)
+{
+  GstRTPMuxPadPrivate *padpriv = data;
+
+  gst_caps_replace (&padpriv->out_caps, NULL);
+  g_slice_free (GstRTPMuxPadPrivate, padpriv);
 }
 
 static void
 gst_rtp_mux_setup_sinkpad (GstRTPMux * rtp_mux, GstPad * sinkpad)
 {
+  GstRTPMuxClass *klass;
   GstRTPMuxPadPrivate *padpriv = g_slice_new0 (GstRTPMuxPadPrivate);
 
+  klass = GST_RTP_MUX_GET_CLASS (rtp_mux);
+
   /* setup some pad functions */
   gst_pad_set_setcaps_function (sinkpad, gst_rtp_mux_setcaps);
   gst_pad_set_getcaps_function (sinkpad, gst_rtp_mux_getcaps);
-  gst_pad_set_chain_function (sinkpad, GST_DEBUG_FUNCPTR (gst_rtp_mux_chain));
-  gst_pad_set_chain_list_function (sinkpad,
-      GST_DEBUG_FUNCPTR (gst_rtp_mux_chain_list));
-  gst_pad_set_event_function (sinkpad,
-      GST_DEBUG_FUNCPTR (gst_rtp_mux_sink_event));
+  if (klass->chain_func)
+    gst_pad_set_chain_function (sinkpad, klass->chain_func);
+  if (klass->sink_event_func)
+    gst_pad_set_event_function (sinkpad, klass->sink_event_func);
 
-
-  gst_segment_init (&padpriv->segment, GST_FORMAT_UNDEFINED);
+  /* This could break with gstreamer 0.10.9 */
+  gst_pad_set_active (sinkpad, TRUE);
 
   gst_pad_set_element_private (sinkpad, padpriv);
+  g_object_weak_ref (G_OBJECT (sinkpad), free_pad_private, padpriv);
 
-  gst_pad_set_active (sinkpad, TRUE);
+  /* dd the pad to the element */
   gst_element_add_pad (GST_ELEMENT (rtp_mux), sinkpad);
 }
 
@@ -295,7 +297,7 @@ gst_rtp_mux_request_new_pad (GstElement * element,
     return NULL;
   }
 
-  newpad = gst_pad_new_from_template (templ, req_name);
+  newpad = gst_rtp_mux_create_sinkpad (rtp_mux, templ);
   if (newpad)
     gst_rtp_mux_setup_sinkpad (rtp_mux, newpad);
   else
@@ -304,33 +306,16 @@ gst_rtp_mux_request_new_pad (GstElement * element,
   return newpad;
 }
 
-static void
-gst_rtp_mux_release_pad (GstElement * element, GstPad * pad)
-{
-  GstRTPMuxPadPrivate *padpriv;
-
-  GST_OBJECT_LOCK (element);
-  padpriv = gst_pad_get_element_private (pad);
-  gst_pad_set_element_private (pad, NULL);
-  GST_OBJECT_UNLOCK (element);
-
-  gst_element_remove_pad (element, pad);
-
-  if (padpriv) {
-    gst_caps_replace (&padpriv->out_caps, NULL);
-    g_slice_free (GstRTPMuxPadPrivate, padpriv);
-  }
-}
-
 /* Put our own clock-base on the buffer */
 static void
-gst_rtp_mux_readjust_rtp_timestamp_locked (GstRTPMux * rtp_mux,
-    GstRTPMuxPadPrivate * padpriv, GstBuffer * buffer)
+gst_rtp_mux_readjust_rtp_timestamp (GstRTPMux * rtp_mux, GstPad * pad,
+    GstBuffer * buffer)
 {
   guint32 ts;
   guint32 sink_ts_base = 0;
+  GstRTPMuxPadPrivate *padpriv = gst_pad_get_element_private (pad);
 
-  if (padpriv && padpriv->have_clock_base)
+  if (padpriv->have_clock_base)
     sink_ts_base = padpriv->clock_base;
 
   ts = gst_rtp_buffer_get_timestamp (buffer) - sink_ts_base + rtp_mux->ts_base;
@@ -339,185 +324,38 @@ gst_rtp_mux_readjust_rtp_timestamp_locked (GstRTPMux * rtp_mux,
   gst_rtp_buffer_set_timestamp (buffer, ts);
 }
 
-static gboolean
-process_buffer_locked (GstRTPMux * rtp_mux, GstRTPMuxPadPrivate * padpriv,
-    GstBuffer * buffer)
-{
-  GstRTPMuxClass *klass = GST_RTP_MUX_GET_CLASS (rtp_mux);
-
-  if (klass->accept_buffer_locked)
-    if (!klass->accept_buffer_locked (rtp_mux, padpriv, buffer))
-      return FALSE;
-
-  rtp_mux->seqnum++;
-  gst_rtp_buffer_set_seq (buffer, rtp_mux->seqnum);
-
-  gst_rtp_buffer_set_ssrc (buffer, rtp_mux->current_ssrc);
-  gst_rtp_mux_readjust_rtp_timestamp_locked (rtp_mux, padpriv, buffer);
-  GST_LOG_OBJECT (rtp_mux, "Pushing packet size %d, seq=%d, ts=%u",
-      GST_BUFFER_SIZE (buffer), rtp_mux->seqnum,
-      gst_rtp_buffer_get_timestamp (buffer));
-
-  if (padpriv) {
-    gst_buffer_set_caps (buffer, padpriv->out_caps);
-    if (padpriv->segment.format == GST_FORMAT_TIME)
-      GST_BUFFER_TIMESTAMP (buffer) =
-          gst_segment_to_running_time (&padpriv->segment, GST_FORMAT_TIME,
-          GST_BUFFER_TIMESTAMP (buffer));
-  }
-
-  return TRUE;
-}
-
-static GstFlowReturn
-gst_rtp_mux_chain_list (GstPad * pad, GstBufferList * bufferlist)
-{
-  GstRTPMux *rtp_mux;
-  GstFlowReturn ret;
-  GstBufferListIterator *it;
-  GstRTPMuxPadPrivate *padpriv;
-  GstEvent *newseg_event = NULL;
-  gboolean drop = TRUE;
-
-  rtp_mux = GST_RTP_MUX (gst_pad_get_parent (pad));
-
-  if (!gst_rtp_buffer_list_validate (bufferlist)) {
-    GST_ERROR_OBJECT (rtp_mux, "Invalid RTP buffer");
-    gst_object_unref (rtp_mux);
-    return GST_FLOW_ERROR;
-  }
-
-  GST_OBJECT_LOCK (rtp_mux);
-
-  padpriv = gst_pad_get_element_private (pad);
-  if (!padpriv) {
-    GST_OBJECT_UNLOCK (rtp_mux);
-    ret = GST_FLOW_NOT_LINKED;
-    gst_buffer_list_unref (bufferlist);
-    goto out;
-  }
-
-  bufferlist = gst_buffer_list_make_writable (bufferlist);
-  it = gst_buffer_list_iterate (bufferlist);
-  while (gst_buffer_list_iterator_next_group (it)) {
-    GstBuffer *rtpbuf;
-
-    rtpbuf = gst_buffer_list_iterator_next (it);
-    rtpbuf = gst_buffer_make_writable (rtpbuf);
-
-    drop = !process_buffer_locked (rtp_mux, padpriv, rtpbuf);
-
-    if (drop)
-      break;
-
-    gst_buffer_list_iterator_take (it, rtpbuf);
-
-    do {
-      if (GST_BUFFER_DURATION_IS_VALID (rtpbuf) &&
-          GST_BUFFER_TIMESTAMP_IS_VALID (rtpbuf))
-        rtp_mux->last_stop = GST_BUFFER_TIMESTAMP (rtpbuf) +
-            GST_BUFFER_DURATION (rtpbuf);
-      else
-        rtp_mux->last_stop = GST_CLOCK_TIME_NONE;
-
-      gst_buffer_list_iterator_take (it, rtpbuf);
-
-    } while ((rtpbuf = gst_buffer_list_iterator_next (it)) != NULL);
-
-
-  }
-  gst_buffer_list_iterator_free (it);
-
-  if (!drop && rtp_mux->segment_pending) {
-    /*
-     * We set the start at 0, because we re-timestamps to the running time
-     */
-    newseg_event = gst_event_new_new_segment_full (FALSE, 1.0, 1.0,
-        GST_FORMAT_TIME, 0, -1, 0);
-
-    rtp_mux->segment_pending = FALSE;
-  }
-
-  GST_OBJECT_UNLOCK (rtp_mux);
-
-  if (newseg_event)
-    gst_pad_push_event (rtp_mux->srcpad, newseg_event);
-
-  if (drop) {
-    gst_buffer_list_unref (bufferlist);
-    ret = GST_FLOW_OK;
-  } else {
-    ret = gst_pad_push_list (rtp_mux->srcpad, bufferlist);
-  }
-
-out:
-
-  gst_object_unref (rtp_mux);
-
-  return ret;
-}
-
 static GstFlowReturn
 gst_rtp_mux_chain (GstPad * pad, GstBuffer * buffer)
 {
   GstRTPMux *rtp_mux;
   GstFlowReturn ret;
-  GstRTPMuxPadPrivate *padpriv;
-  GstEvent *newseg_event = NULL;
-  gboolean drop;
+  GstRTPMuxPadPrivate *padpriv = gst_pad_get_element_private (pad);
 
-  rtp_mux = GST_RTP_MUX (GST_OBJECT_PARENT (pad));
+  rtp_mux = GST_RTP_MUX (gst_pad_get_parent (pad));
 
   if (!gst_rtp_buffer_validate (buffer)) {
-    gst_buffer_unref (buffer);
     GST_ERROR_OBJECT (rtp_mux, "Invalid RTP buffer");
+    gst_object_unref (rtp_mux);
     return GST_FLOW_ERROR;
   }
 
-  GST_OBJECT_LOCK (rtp_mux);
-  padpriv = gst_pad_get_element_private (pad);
-
-  if (!padpriv) {
-    GST_OBJECT_UNLOCK (rtp_mux);
-    gst_buffer_unref (buffer);
-    return GST_FLOW_NOT_LINKED;
-  }
-
   buffer = gst_buffer_make_writable (buffer);
 
-  drop = !process_buffer_locked (rtp_mux, padpriv, buffer);
-
-  if (!drop) {
-    if (rtp_mux->segment_pending) {
-      /*
-       * We set the start at 0, because we re-timestamps to the running time
-       */
-      newseg_event = gst_event_new_new_segment_full (FALSE, 1.0, 1.0,
-          GST_FORMAT_TIME, 0, -1, 0);
-
-      rtp_mux->segment_pending = FALSE;
-    }
-
-    if (GST_BUFFER_DURATION_IS_VALID (buffer) &&
-        GST_BUFFER_TIMESTAMP_IS_VALID (buffer))
-      rtp_mux->last_stop = GST_BUFFER_TIMESTAMP (buffer) +
-          GST_BUFFER_DURATION (buffer);
-    else
-      rtp_mux->last_stop = GST_CLOCK_TIME_NONE;
-  }
-
+  GST_OBJECT_LOCK (rtp_mux);
+  rtp_mux->seqnum++;
+  gst_rtp_buffer_set_seq (buffer, rtp_mux->seqnum);
   GST_OBJECT_UNLOCK (rtp_mux);
+  gst_rtp_buffer_set_ssrc (buffer, rtp_mux->current_ssrc);
+  gst_rtp_mux_readjust_rtp_timestamp (rtp_mux, pad, buffer);
+  GST_LOG_OBJECT (rtp_mux, "Pushing packet size %d, seq=%d, ts=%u",
+      GST_BUFFER_SIZE (buffer), rtp_mux->seqnum,
+      gst_rtp_buffer_get_timestamp (buffer));
 
-  if (newseg_event)
-    gst_pad_push_event (rtp_mux->srcpad, newseg_event);
+  gst_buffer_set_caps (buffer, padpriv->out_caps);
 
-  if (drop) {
-    gst_buffer_unref (buffer);
-    ret = GST_FLOW_OK;
-  } else {
-    ret = gst_pad_push (rtp_mux->srcpad, buffer);
-  }
+  ret = gst_pad_push (rtp_mux->srcpad, buffer);
 
+  gst_object_unref (rtp_mux);
   return ret;
 }
 
@@ -527,7 +365,7 @@ gst_rtp_mux_setcaps (GstPad * pad, GstCaps * caps)
   GstRTPMux *rtp_mux;
   GstStructure *structure;
   gboolean ret = FALSE;
-  GstRTPMuxPadPrivate *padpriv;
+  GstRTPMuxPadPrivate *padpriv = gst_pad_get_element_private (pad);
 
   rtp_mux = GST_RTP_MUX (gst_pad_get_parent (pad));
 
@@ -536,13 +374,9 @@ gst_rtp_mux_setcaps (GstPad * pad, GstCaps * caps)
   if (!structure)
     goto out;
 
-  GST_OBJECT_LOCK (rtp_mux);
-  padpriv = gst_pad_get_element_private (pad);
-  if (padpriv &&
-      gst_structure_get_uint (structure, "clock-base", &padpriv->clock_base)) {
+  if (gst_structure_get_uint (structure, "clock-base", &padpriv->clock_base)) {
     padpriv->have_clock_base = TRUE;
   }
-  GST_OBJECT_UNLOCK (rtp_mux);
 
   caps = gst_caps_copy (caps);
 
@@ -554,20 +388,8 @@ gst_rtp_mux_setcaps (GstPad * pad, GstCaps * caps)
       "setting caps %" GST_PTR_FORMAT " on src pad..", caps);
   ret = gst_pad_set_caps (rtp_mux->srcpad, caps);
 
-  if (rtp_mux->ssrc == -1) {
-    if (gst_structure_has_field_typed (structure, "ssrc", G_TYPE_UINT)) {
-      rtp_mux->current_ssrc = g_value_get_uint
-          (gst_structure_get_value (structure, "ssrc"));
-    }
-  }
-
-  if (ret) {
-    GST_OBJECT_LOCK (rtp_mux);
-    padpriv = gst_pad_get_element_private (pad);
-    if (padpriv)
-      gst_caps_replace (&padpriv->out_caps, caps);
-    GST_OBJECT_UNLOCK (rtp_mux);
-  }
+  if (ret)
+    gst_caps_replace (&padpriv->out_caps, caps);
   gst_caps_unref (caps);
 
 out:
@@ -734,98 +556,10 @@ gst_rtp_mux_set_property (GObject * object,
   }
 }
 
-static gboolean
-gst_rtp_mux_sink_event (GstPad * pad, GstEvent * event)
-{
-
-  GstRTPMux *mux;
-  gboolean ret = FALSE;
-  gboolean forward = TRUE;
-
-  mux = GST_RTP_MUX (gst_pad_get_parent (pad));
-
-  switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_FLUSH_STOP:
-    {
-      GstRTPMuxPadPrivate *padpriv;
-
-      GST_OBJECT_LOCK (mux);
-      mux->last_stop = GST_CLOCK_TIME_NONE;
-      mux->segment_pending = TRUE;
-      padpriv = gst_pad_get_element_private (pad);
-      if (padpriv)
-        gst_segment_init (&padpriv->segment, GST_FORMAT_UNDEFINED);
-      GST_OBJECT_UNLOCK (mux);
-    }
-      break;
-    case GST_EVENT_NEWSEGMENT:
-    {
-      gboolean update;
-      gdouble rate, applied_rate;
-      GstFormat format;
-      gint64 start, stop, position;
-      GstRTPMuxPadPrivate *padpriv;
-
-      gst_event_parse_new_segment_full (event, &update, &rate, &applied_rate,
-          &format, &start, &stop, &position);
-
-      GST_OBJECT_LOCK (mux);
-      padpriv = gst_pad_get_element_private (pad);
-
-      if (padpriv) {
-        if (format == GST_FORMAT_TIME)
-          gst_segment_set_newsegment_full (&padpriv->segment, update,
-              rate, applied_rate, format, start, stop, position);
-        else
-          gst_segment_init (&padpriv->segment, GST_FORMAT_UNDEFINED);
-      }
-      GST_OBJECT_UNLOCK (mux);
-      gst_event_unref (event);
-      forward = FALSE;
-      ret = TRUE;
-      break;
-    }
-    default:
-      break;
-  }
-
-  if (forward)
-    ret = gst_pad_push_event (mux->srcpad, event);
-
-  gst_object_unref (mux);
-  return ret;
-}
-
-
-static void
-clear_segment (gpointer data, gpointer user_data)
-{
-  GstPad *pad = data;
-  GstRTPMux *mux = user_data;
-  GstRTPMuxPadPrivate *padpriv;
-
-  GST_OBJECT_LOCK (mux);
-  padpriv = gst_pad_get_element_private (pad);
-  if (padpriv)
-    gst_segment_init (&padpriv->segment, GST_FORMAT_UNDEFINED);
-  GST_OBJECT_UNLOCK (mux);
-
-  gst_object_unref (pad);
-}
-
-
 static void
 gst_rtp_mux_ready_to_paused (GstRTPMux * rtp_mux)
 {
-  GstIterator *iter;
-
-  iter = gst_element_iterate_sink_pads (GST_ELEMENT (rtp_mux));
-  while (gst_iterator_foreach (iter, clear_segment, rtp_mux) ==
-      GST_ITERATOR_RESYNC);
-  gst_iterator_free (iter);
-
   GST_OBJECT_LOCK (rtp_mux);
-  rtp_mux->segment_pending = TRUE;
 
   if (rtp_mux->ssrc == -1)
     rtp_mux->current_ssrc = g_random_int ();
@@ -842,9 +576,6 @@ gst_rtp_mux_ready_to_paused (GstRTPMux * rtp_mux)
     rtp_mux->ts_base = g_random_int ();
   else
     rtp_mux->ts_base = rtp_mux->ts_offset;
-
-  rtp_mux->last_stop = GST_CLOCK_TIME_NONE;
-
   GST_DEBUG_OBJECT (rtp_mux, "set clock-base to %u", rtp_mux->ts_base);
 
   GST_OBJECT_UNLOCK (rtp_mux);
