@@ -109,7 +109,7 @@ acmenc_caps_from_format (WAVEFORMATEX * fmt)
       (gst_riff_strf_auds *) fmt, NULL, NULL, NULL);
 }
 
-static gboolean
+gboolean
 acmenc_set_input_format (ACMEnc * enc, WAVEFORMATEX * infmt)
 {
   infmt->wFormatTag = WAVE_FORMAT_PCM;
@@ -122,7 +122,7 @@ acmenc_set_input_format (ACMEnc * enc, WAVEFORMATEX * infmt)
   return TRUE;
 }
 
-static BOOL CALLBACK
+BOOL CALLBACK
 acmenc_format_enum (HACMDRIVERID driverId, LPACMFORMATDETAILS fd,
     DWORD_PTR dwInstance, DWORD fdwSupport)
 {
@@ -160,7 +160,7 @@ acmenc_format_enum (HACMDRIVERID driverId, LPACMFORMATDETAILS fd,
   return TRUE;
 }
 
-static gboolean
+gboolean
 acmenc_set_format (ACMEnc * enc)
 {
   WAVEFORMATEX *in = NULL;
@@ -310,7 +310,7 @@ acmenc_push_output (ACMEnc * enc)
           enc->outfmt->nAvgBytesPerSec);
     }
     enc->bytes_output += enc->header.cbDstLengthUsed;
-    GST_DEBUG_OBJECT (enc, "Pushing %lu byte encoded buffer",
+    GST_DEBUG_OBJECT (enc, "Pushing %d byte encoded buffer",
         enc->header.cbDstLengthUsed);
     ret = gst_pad_push (enc->srcpad, outbuf);
   }
@@ -361,7 +361,7 @@ acmenc_chain (GstPad * pad, GstBuffer * buf)
   return ret;
 }
 
-static GstFlowReturn
+GstFlowReturn
 acmenc_finish_stream (ACMEnc * enc)
 {
   MMRESULT res;
@@ -465,15 +465,15 @@ acmenc_class_init (ACMEncClass * klass)
 acmenc_base_init (ACMEncClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+  GstElementDetails details;
   ACMEncParams *params;
   ACMDRIVERDETAILS driverdetails;
-  gchar *shortname, *longname, *detail, *description;
+  gchar *shortname, *longname;
   MMRESULT res;
-
-  gst_element_class_add_static_pad_template (element_class,
-      &acmenc_sink_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &acmenc_src_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&acmenc_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&acmenc_src_template));
   params =
       (ACMEncParams *) g_type_get_qdata (G_OBJECT_CLASS_TYPE (klass),
       ACMENC_PARAMS_QDATA);
@@ -490,17 +490,17 @@ acmenc_base_init (ACMEncClass * klass)
   longname =
       g_utf16_to_utf8 ((gunichar2 *) driverdetails.szLongName, -1, NULL, NULL,
       NULL);
-  detail = g_strdup_printf ("ACM Encoder: %s", (shortname
+  details.longname = g_strdup_printf ("ACM Encoder: %s", (shortname
           && *shortname) ? shortname : params->name);
-  description = g_strdup_printf ("ACM Encoder: %s", (longname
+  details.klass = "Codec/Encoder/Audio";
+  details.description = g_strdup_printf ("ACM Encoder: %s", (longname
           && *longname) ? longname : params->name);
-  gst_element_class_set_details_simple (element_class, detail,
-      "Codec/Encoder/Audio", description,
-      "Pioneers of the Inevitable <songbird@songbirdnest.com>");
+  details.author = "Pioneers of the Inevitable <songbird@songbirdnest.com>";
+  gst_element_class_set_details (element_class, &details);
   g_free (shortname);
   g_free (longname);
-  g_free (description);
-  g_free (detail);
+  g_free (details.longname);
+  g_free (details.description);
   klass->driverId = params->driverId;
 }
 
@@ -594,7 +594,7 @@ acmenc_register (GstPlugin * plugin)
   wcscat (dirname, L"\\*.acm");
   find = FindFirstFileW (dirname, &filedata);
   if (find == INVALID_HANDLE_VALUE) {
-    GST_WARNING ("Failed to find ACM files: %lx", GetLastError ());
+    GST_WARNING ("Failed to find ACM files: %x", GetLastError ());
     return FALSE;
   }
 
