@@ -1,6 +1,5 @@
 /*
  * Copyright (C) 2008 Ole André Vadla Ravnås <ole.andre.ravnas@tandberg.com>
- *               2009 Andres Colubri <andres.colubri@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -84,7 +83,7 @@ typedef enum
   KS_WORKER_STATE_ERROR,
 } KsWorkerState;
 
-struct _GstKsVideoSrcPrivate
+typedef struct
 {
   /* Properties */
   gchar *device_path;
@@ -114,17 +113,15 @@ struct _GstKsVideoSrcPrivate
   gboolean worker_pending_run;
   gboolean worker_run_result;
 
-  gulong worker_error_code;
-
   /* Statistics */
   GstClockTime last_sampling;
   guint count;
   guint fps;
-};
+} GstKsVideoSrcPrivate;
 
-#define GST_KS_VIDEO_SRC_GET_PRIVATE(o) ((o)->priv)
-
-static void gst_ks_video_src_init_interfaces (GType type);
+#define GST_KS_VIDEO_SRC_GET_PRIVATE(o) \
+    (G_TYPE_INSTANCE_GET_PRIVATE ((o), GST_TYPE_KS_VIDEO_SRC, \
+    GstKsVideoSrcPrivate))
 
 static void gst_ks_video_src_finalize (GObject * object);
 static void gst_ks_video_src_get_property (GObject * object, guint prop_id,
@@ -132,14 +129,6 @@ static void gst_ks_video_src_get_property (GObject * object, guint prop_id,
 static void gst_ks_video_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 
-static void gst_ks_video_src_probe_interface_init (GstPropertyProbeInterface *
-    iface);
-static const GList *gst_ks_video_src_probe_get_properties (GstPropertyProbe *
-    probe);
-static GValueArray *gst_ks_video_src_probe_get_values (GstPropertyProbe * probe,
-    guint prop_id, const GParamSpec * pspec);
-static GValueArray *gst_ks_video_src_get_device_name_values (GstKsVideoSrc *
-    self);
 static void gst_ks_video_src_reset (GstKsVideoSrc * self);
 
 static GstStateChangeReturn gst_ks_video_src_change_state (GstElement * element,
@@ -157,30 +146,27 @@ static gboolean gst_ks_video_src_unlock_stop (GstBaseSrc * basesrc);
 
 static GstFlowReturn gst_ks_video_src_create (GstPushSrc * pushsrc,
     GstBuffer ** buffer);
-static GstBuffer *gst_ks_video_src_alloc_buffer (guint size, guint alignment,
-    gpointer user_data);
 
-GST_BOILERPLATE_FULL (GstKsVideoSrc, gst_ks_video_src, GstPushSrc,
-    GST_TYPE_PUSH_SRC, gst_ks_video_src_init_interfaces);
+GST_BOILERPLATE (GstKsVideoSrc, gst_ks_video_src, GstPushSrc,
+    GST_TYPE_PUSH_SRC);
 
 static void
 gst_ks_video_src_base_init (gpointer gclass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (gclass);
-  GstPadTemplate *pad_template;
+  static GstElementDetails element_details = {
+    "KsVideoSrc",
+    "Source/Video",
+    "Stream data from a video capture device through Windows kernel streaming",
+    "Ole André Vadla Ravnås <ole.andre.ravnas@tandberg.com>\n"
+        "Haakon Sporsheim <hakon.sporsheim@tandberg.com>"
+  };
 
-  gst_element_class_set_details_simple (element_class, "KsVideoSrc",
-      "Source/Video",
-      "Stream data from a video capture device through Windows kernel streaming",
-      "Ole André Vadla Ravnås <ole.andre.ravnas@tandberg.com>\n"
-      "Haakon Sporsheim <hakon.sporsheim@tandberg.com>\n"
-      "Andres Colubri <andres.colubri@gmail.com>");
+  gst_element_class_set_details (element_class, &element_details);
 
-  pad_template =
+  gst_element_class_add_pad_template (element_class,
       gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS,
-      ks_video_get_all_caps ());
-  gst_element_class_add_pad_template (element_class, pad_template);
-  gst_object_unref (pad_template);
+          ks_video_get_all_caps ()));
 }
 
 static void
@@ -243,12 +229,8 @@ gst_ks_video_src_class_init (GstKsVideoSrcClass * klass)
 static void
 gst_ks_video_src_init (GstKsVideoSrc * self, GstKsVideoSrcClass * gclass)
 {
+  GstKsVideoSrcPrivate *priv = GST_KS_VIDEO_SRC_GET_PRIVATE (self);
   GstBaseSrc *basesrc = GST_BASE_SRC (self);
-  GstKsVideoSrcPrivate *priv;
-
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GST_TYPE_KS_VIDEO_SRC,
-      GstKsVideoSrcPrivate);
-  priv = GST_KS_VIDEO_SRC_GET_PRIVATE (self);
 
   gst_base_src_set_live (basesrc, TRUE);
   gst_base_src_set_format (basesrc, GST_FORMAT_TIME);
@@ -323,13 +305,8 @@ gst_ks_video_src_set_property (GObject * object, guint prop_id,
       priv->device_path = g_value_dup_string (value);
       break;
     case PROP_DEVICE_NAME:
-    {
-      const gchar *device_name = g_value_get_string (value);
       g_free (priv->device_name);
-      priv->device_name = NULL;
-      if (device_name && strlen (device_name) != 0)
-        priv->device_name = g_strdup (device_name);
-    }
+      priv->device_name = g_value_dup_string (value);
       break;
     case PROP_DEVICE_INDEX:
       priv->device_index = g_value_get_int (value);
@@ -401,88 +378,6 @@ gst_ks_video_src_apply_driver_quirks (GstKsVideoSrc * self)
   }
 }
 
-static void
-gst_ks_video_src_init_interfaces (GType type)
-{
-  static const GInterfaceInfo ksvideosrc_info = {
-    (GInterfaceInitFunc) gst_ks_video_src_probe_interface_init,
-    NULL,
-    NULL,
-  };
-
-  g_type_add_interface_static (type, GST_TYPE_PROPERTY_PROBE, &ksvideosrc_info);
-}
-
-static void
-gst_ks_video_src_probe_interface_init (GstPropertyProbeInterface * iface)
-{
-  iface->get_properties = gst_ks_video_src_probe_get_properties;
-  iface->get_values = gst_ks_video_src_probe_get_values;
-}
-
-static const GList *
-gst_ks_video_src_probe_get_properties (GstPropertyProbe * probe)
-{
-  GObjectClass *klass = G_OBJECT_GET_CLASS (probe);
-  static GList *props = NULL;
-
-  if (!props) {
-    GParamSpec *pspec;
-
-    pspec = g_object_class_find_property (klass, "device-name");
-    props = g_list_append (props, pspec);
-  }
-
-  return props;
-}
-
-static GValueArray *
-gst_ks_video_src_probe_get_values (GstPropertyProbe * probe, guint prop_id,
-    const GParamSpec * pspec)
-{
-  GstKsVideoSrc *src = GST_KS_VIDEO_SRC (probe);
-  GValueArray *array = NULL;
-
-  switch (prop_id) {
-    case PROP_DEVICE_NAME:
-      array = gst_ks_video_src_get_device_name_values (src);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (probe, prop_id, pspec);
-      break;
-  }
-
-  return array;
-}
-
-static GValueArray *
-gst_ks_video_src_get_device_name_values (GstKsVideoSrc * self)
-{
-  GList *devices, *cur;
-  GValueArray *array = g_value_array_new (0);
-
-  devices = ks_enumerate_devices (&KSCATEGORY_VIDEO);
-  if (devices == NULL)
-    return array;
-
-  devices = ks_video_device_list_sort_cameras_first (devices);
-
-  for (cur = devices; cur != NULL; cur = cur->next) {
-    GValue value = { 0, };
-    KsDeviceEntry *entry = cur->data;
-
-    g_value_init (&value, G_TYPE_STRING);
-    g_value_set_string (&value, entry->name);
-    g_value_array_append (array, &value);
-    g_value_unset (&value);
-
-    ks_device_entry_free (entry);
-  }
-
-  g_list_free (devices);
-  return array;
-}
-
 static gboolean
 gst_ks_video_src_open_device (GstKsVideoSrc * self)
 {
@@ -496,8 +391,6 @@ gst_ks_video_src_open_device (GstKsVideoSrc * self)
   if (devices == NULL)
     goto error_no_devices;
 
-  devices = ks_video_device_list_sort_cameras_first (devices);
-
   for (cur = devices; cur != NULL; cur = cur->next) {
     KsDeviceEntry *entry = cur->data;
 
@@ -531,8 +424,8 @@ gst_ks_video_src_open_device (GstKsVideoSrc * self)
         priv->ksclock = NULL;
       }
 
-      device = gst_ks_video_device_new (entry->path, priv->ksclock,
-          gst_ks_video_src_alloc_buffer, self);
+      device = g_object_new (GST_TYPE_KS_VIDEO_DEVICE,
+          "clock", priv->ksclock, "device-path", entry->path, NULL);
     }
 
     ks_device_entry_free (entry);
@@ -560,15 +453,15 @@ error_no_devices:
 error_no_match:
   {
     if (priv->device_path != NULL) {
-      GST_ELEMENT_ERROR (self, RESOURCE, NOT_FOUND,
+      GST_ELEMENT_ERROR (self, RESOURCE, SETTINGS,
           ("Specified video capture device with path '%s' not found",
               priv->device_path), (NULL));
     } else if (priv->device_name != NULL) {
-      GST_ELEMENT_ERROR (self, RESOURCE, NOT_FOUND,
+      GST_ELEMENT_ERROR (self, RESOURCE, SETTINGS,
           ("Specified video capture device with name '%s' not found",
               priv->device_name), (NULL));
     } else {
-      GST_ELEMENT_ERROR (self, RESOURCE, NOT_FOUND,
+      GST_ELEMENT_ERROR (self, RESOURCE, SETTINGS,
           ("Specified video capture device with index %d not found",
               priv->device_index), (NULL));
     }
@@ -638,8 +531,8 @@ gst_ks_video_src_worker_func (gpointer data)
     } else if (priv->worker_pending_run) {
       if (priv->ksclock != NULL)
         gst_ks_clock_start (priv->ksclock);
-      priv->worker_run_result = gst_ks_video_device_set_state (priv->device,
-          KSSTATE_RUN, &priv->worker_error_code);
+      priv->worker_run_result =
+          gst_ks_video_device_set_state (priv->device, KSSTATE_RUN);
 
       priv->worker_pending_run = FALSE;
       KS_WORKER_NOTIFY_RESULT (priv);
@@ -919,7 +812,7 @@ gst_ks_video_src_timestamp_buffer (GstKsVideoSrc * self, GstBuffer * buf,
       /* REVISIT: I've seen this happen with the GstSystemClock on Windows,
        *          scary... */
       if (timestamp < priv->prev_ts) {
-        GST_INFO_OBJECT (self, "clock is ticking backwards");
+        GST_WARNING_OBJECT (self, "clock is ticking backwards");
         return FALSE;
       }
 
@@ -998,10 +891,13 @@ gst_ks_video_src_update_statistics (GstKsVideoSrc * self)
 }
 
 static GstFlowReturn
-gst_ks_video_src_create (GstPushSrc * pushsrc, GstBuffer ** buf)
+gst_ks_video_src_create (GstPushSrc * pushsrc, GstBuffer ** buffer)
 {
   GstKsVideoSrc *self = GST_KS_VIDEO_SRC (pushsrc);
   GstKsVideoSrcPrivate *priv = GST_KS_VIDEO_SRC_GET_PRIVATE (self);
+  guint buf_size;
+  GstCaps *caps;
+  GstBuffer *buf = NULL;
   GstFlowReturn result;
   GstClockTime presentation_time;
   gulong error_code;
@@ -1012,6 +908,18 @@ gst_ks_video_src_create (GstPushSrc * pushsrc, GstBuffer ** buf)
   if (!gst_ks_video_device_has_caps (priv->device))
     goto error_no_caps;
 
+  buf_size = gst_ks_video_device_get_frame_size (priv->device);
+  g_assert (buf_size);
+
+  caps = gst_pad_get_negotiated_caps (GST_BASE_SRC_PAD (self));
+  if (caps == NULL)
+    goto error_no_caps;
+  result = gst_pad_alloc_buffer (GST_BASE_SRC_PAD (self), priv->offset,
+      buf_size, caps, &buf);
+  gst_caps_unref (caps);
+  if (G_UNLIKELY (result != GST_FLOW_OK))
+    goto error_alloc_buffer;
+
   if (G_UNLIKELY (!priv->running)) {
     KS_WORKER_LOCK (priv);
     priv->worker_pending_run = TRUE;
@@ -1019,7 +927,6 @@ gst_ks_video_src_create (GstPushSrc * pushsrc, GstBuffer ** buf)
     while (priv->worker_pending_run)
       KS_WORKER_WAIT_FOR_RESULT (priv);
     priv->running = priv->worker_run_result;
-    error_code = priv->worker_error_code;
     KS_WORKER_UNLOCK (priv);
 
     if (!priv->running)
@@ -1027,24 +934,25 @@ gst_ks_video_src_create (GstPushSrc * pushsrc, GstBuffer ** buf)
   }
 
   do {
-    if (*buf != NULL) {
-      gst_buffer_unref (*buf);
-      *buf = NULL;
-    }
+    gulong bytes_read;
 
-    result = gst_ks_video_device_read_frame (priv->device, buf,
-        &presentation_time, &error_code, &error_str);
+    result = gst_ks_video_device_read_frame (priv->device,
+        GST_BUFFER_DATA (buf), buf_size, &bytes_read, &presentation_time,
+        &error_code, &error_str);
     if (G_UNLIKELY (result != GST_FLOW_OK))
       goto error_read_frame;
+
+    GST_BUFFER_SIZE (buf) = bytes_read;
   }
-  while (!gst_ks_video_src_timestamp_buffer (self, *buf, presentation_time));
+  while (!gst_ks_video_src_timestamp_buffer (self, buf, presentation_time));
 
   if (G_UNLIKELY (priv->do_stats))
     gst_ks_video_src_update_statistics (self);
 
   gst_ks_video_device_postprocess_frame (priv->device,
-      GST_BUFFER_DATA (*buf), GST_BUFFER_SIZE (*buf));
+      GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
 
+  *buffer = buf;
   return GST_FLOW_OK;
 
   /* ERRORS */
@@ -1057,80 +965,30 @@ error_no_caps:
   }
 error_start_capture:
   {
-    const gchar *debug_str = "failed to change pin state to KSSTATE_RUN";
-
-    switch (error_code) {
-      case ERROR_FILE_NOT_FOUND:
-        GST_ELEMENT_ERROR (self, RESOURCE, NOT_FOUND,
-            ("failed to start capture (device unplugged)"), (debug_str));
-        break;
-      case ERROR_NO_SYSTEM_RESOURCES:
-        GST_ELEMENT_ERROR (self, RESOURCE, BUSY,
-            ("failed to start capture (device already in use)"), (debug_str));
-        break;
-      default:
-        GST_ELEMENT_ERROR (self, RESOURCE, FAILED,
-            ("failed to start capture (0x%08x)", error_code), (debug_str));
-        break;
-    }
+    GST_ELEMENT_ERROR (self, RESOURCE, OPEN_READ,
+        ("could not start capture"),
+        ("failed to change pin state to KSSTATE_RUN"));
 
     return GST_FLOW_ERROR;
   }
-error_read_frame:
-  {
-    if (result == GST_FLOW_ERROR) {
-      if (error_str != NULL) {
-        GST_ELEMENT_ERROR (self, RESOURCE, READ,
-            ("read failed: %s [0x%08x]", error_str, error_code),
-            ("gst_ks_video_device_read_frame failed"));
-      }
-    } else if (result == GST_FLOW_UNEXPECTED) {
-      GST_ELEMENT_ERROR (self, RESOURCE, READ,
-          ("read failed"), ("gst_ks_video_device_read_frame failed"));
-    }
-
-    g_free (error_str);
-
-    return result;
-  }
-}
-
-static GstBuffer *
-gst_ks_video_src_alloc_buffer (guint size, guint alignment, gpointer user_data)
-{
-  GstKsVideoSrc *self = GST_KS_VIDEO_SRC (user_data);
-  GstBuffer *buf;
-  GstCaps *caps;
-  GstFlowReturn flow_ret;
-
-  caps = gst_pad_get_negotiated_caps (GST_BASE_SRC_PAD (self));
-  if (caps == NULL)
-    goto error_no_caps;
-  flow_ret = gst_pad_alloc_buffer (GST_BASE_SRC_PAD (self), 0,
-      size + (alignment - 1), caps, &buf);
-  gst_caps_unref (caps);
-  if (G_UNLIKELY (flow_ret != GST_FLOW_OK))
-    goto error_alloc_buffer;
-
-  GST_BUFFER_DATA (buf) =
-      GSIZE_TO_POINTER ((GPOINTER_TO_SIZE (GST_BUFFER_DATA (buf)) + (alignment -
-              1)) & ~(alignment - 1));
-  GST_BUFFER_SIZE (buf) = size;
-
-  return buf;
-
-error_no_caps:
-  {
-    GST_ELEMENT_ERROR (self, CORE, NEGOTIATION,
-        ("not negotiated"), ("maybe setcaps failed?"));
-
-    return NULL;
-  }
 error_alloc_buffer:
   {
     GST_ELEMENT_ERROR (self, CORE, PAD, ("alloc_buffer failed"), (NULL));
 
-    return NULL;
+    return result;
+  }
+error_read_frame:
+  {
+    if (result != GST_FLOW_WRONG_STATE && result != GST_FLOW_UNEXPECTED) {
+      GST_ELEMENT_ERROR (self, RESOURCE, READ,
+          ("read failed: %s [0x%08x]", error_str, error_code),
+          ("gst_ks_video_device_read_frame failed"));
+    }
+
+    g_free (error_str);
+    gst_buffer_unref (buf);
+
+    return result;
   }
 }
 
